#!/usr/bin/env python
# -*- coding: utf-8 -*-

import logging
import sys
from silx.gui import qt
import argparse
from tomwer.gui import icons
from tomwer.gui.utils.splashscreen import getMainSplashScreen
from tomwer.core.scan.scanfactory import ScanFactory
from tomwer.gui.visualization.diffviewer import DiffFrameViewer
from tomwer.core.utils.resource import increase_max_number_file
import signal

logging.basicConfig(level=logging.WARNING)
_logger = logging.getLogger(__name__)


def getinputinfo():
    return "tomwer nabu [scan_path]"


def sigintHandler(*args):
    """Handler for the SIGINT signal."""
    qt.QApplication.quit()


def main(argv):
    import os

    parser = argparse.ArgumentParser(description=__doc__)
    parser.add_argument(
        "scan_path", help="Data file to show (h5 file, edf files, spec files)"
    )
    parser.add_argument(
        "--entry", default=None, help="An entry should be specify for hdf5 files"
    )
    parser.add_argument("--second-scan", default=None, help="Second data files")
    parser.add_argument("--second-entry", default=None, help="Second entry")
    options = parser.parse_args(argv[1:])
    options.scan_path = options.scan_path.rstrip(os.path.sep)

    if options.second_entry is not None and options.second_scan is None:
        options.second_scan = options.scan_path

    increase_max_number_file()

    app = qt.QApplication.instance() or qt.QApplication([])

    qt.QLocale.setDefault(qt.QLocale(qt.QLocale.English))
    qt.QLocale.setDefault(qt.QLocale.c())
    signal.signal(signal.SIGINT, sigintHandler)
    sys.excepthook = qt.exceptionHandler
    timer = qt.QTimer()
    timer.start(500)
    # Application have to wake up Python interpreter, else SIGINT is not
    # catch
    timer.timeout.connect(lambda: None)

    splash = getMainSplashScreen()

    # create scan
    scan_1 = ScanFactory.create_scan_object(options.scan_path, entry=options.entry)
    if scan_1 is None:
        raise ValueError(
            "Given scan path is not recognized as a path" "containing a scan"
        )
    if options.second_scan is not None:
        scan_2 = ScanFactory.create_scan_object(
            options.second_scan, entry=options.second_entry
        )
    else:
        scan_2 = None

    # handle gui
    widget = DiffFrameViewer(parent=None)
    widget.addScan(scan_1)
    if scan_2 is not None:
        widget.addScan(scan_2)
        widget.setRightScan(scan_2)
    # for the application we run for the reconstruction to be finished
    # to give back hand to the user
    widget.setWindowTitle("Frame diff")
    widget.setWindowIcon(icons.getQIcon("tomwer"))
    splash.finish(widget)
    widget.show()
    app.exec_()


if __name__ == "__main__":
    main(sys.argv)



# coding: utf-8
# /*##########################################################################
#
# Copyright (c) 2016-2017 European Synchrotron Radiation Facility
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# ###########################################################################*/


from .base import _ReconsParam
from .axis import AxisRP
from .beamgeo import BeamGeoRP
from .ft import FTRP
from .paganin import PaganinRP
from .pyhst import PyhstRP


class ReconsParams(_ReconsParam):
    """This class defines the reconstructions parameters to be used for
    tomography.

    Those are meant to be embeded in TomoBase class.
    They are used to tune processes like ftseries or darkref.
    They can be modify by processes (axis, darkref, ftseries).

    In the following scheme for example:

    .. image:: img/orange_data_recons_params.png

    * Data watcher will generate instance of TomoBase (alias data in Orange)
        that will have no reconstruction parameters.
    * Then dark and flat field construction will copy the currently defined 'dkrf'
        reconstruction parameters into each received TomoBase and use them to run
        dark and flat field.
    * Now TomoBase as a definition for dark and flat field and move to axis
        definition.
    * axis process will also copy it's definition of the axis the TomoBase
        reconstruction parameters and process axis process. Process result
        (position_value) will be updated in the TomoBase reconstruction paramters.
    * ftseries load reconstruction paramters of axis and dark and flat field
        (if any) and run the reconstruction.

    :param bool empty: if empty, skip reconstruction of sub parameter object
                       and set them to None instead
    """

    def __init__(self, empty=False):
        _ReconsParam.__init__(self)
        self._createSubParamsSet(empty)

        self._managed_params = {
            "FT": self.__class__.ft,
            "PYHSTEXE": self.__class__.pyhst,
            "FTAXIS": self.__class__.axis,
            "PAGANIN": self.__class__.paganin,
            "BEAMGEO": self.__class__.beam_geo,
            "DKRF": self.__class__.dkrf,
        }

    def _createSubParamsSet(self, empty):
        self._ft = None if empty else FTRP()
        self._pyhst = None if empty else PyhstRP()
        self._axis = None if empty else AxisRP()
        self._paganin = None if empty else PaganinRP()
        self._beam_geo = None if empty else BeamGeoRP()
        from tomwer.core.process.reconstruction.darkref.params import DKRFRP

        self._dkrf = None if empty else DKRFRP()

    def unmanaged_params_tree(self):
        """

        :return: unmanged paramters of the Reconsparams and the one
                 of his 'childrens': FTRP, PyhstRP...
        :rtype: dict
        """
        this_node_unmanged = self.unmanaged_params
        for managed_key in self._managed_params:
            unmanaged = self._managed_params[managed_key].fget(self).unmanaged_params
            if len(unmanaged) > 0:
                if managed_key not in this_node_unmanged:
                    this_node_unmanged[managed_key] = {}
                this_node_unmanged[managed_key].update(unmanaged)
        return this_node_unmanged

    @property
    def ft(self):
        """set of general reconstruction parameters

        :rtype: :class:`.FTRP`
        """
        return self._ft

    @ft.setter
    def ft(self, ft):
        self._ft = ft
        self.changed()

    @property
    def pyhst(self):
        """set of reconstruction parameters regarding pyhst option (version...)

        :rtype: :class:`.PyhstRP`
        """
        return self._pyhst

    @pyhst.setter
    def pyhst(self, pyhst):
        self._pyhst = pyhst
        self.changed()

    @property
    def axis(self):
        """reconstruction parameters regarding the center of rotation

        :rtype: :class:`.AxisRP`
        """
        return self._axis

    @axis.setter
    def axis(self, axis):
        self._axis = axis
        self.changed()

    @property
    def paganin(self):
        """set of paramrters regarding Paganin mode

        :rtype: :class:`.PaganinRP`
        """
        return self._paganin

    @paganin.setter
    def paganin(self, paganin):
        self._paganin = paganin
        self.changed()

    @property
    def beam_geo(self):
        """set of parameters regarding the beam geometry

        :rtype: :class:`.BeamGeoRP`
        """
        return self._beam_geo

    @beam_geo.setter
    def beam_geo(self, beam_geo):
        self._beam_geo = beam_geo
        self.changed()

    @property
    def dkrf(self):
        """set of parameters regarding the dark and flat field correction

        :rtype: :class:`.DKRFRP`
        """
        return self._dkrf

    @dkrf.setter
    def dkrf(self, dkrf):
        self._dkrf = dkrf
        self.changed()

    def to_dict(self):
        _dict = {
            "FT": self.ft.to_dict() if self.ft else None,
            "PYHSTEXE": self.pyhst.to_dict() if self.pyhst else None,
            "FTAXIS": self.axis.to_dict() if self.axis else None,
            "PAGANIN": self.paganin.to_dict() if self.paganin else None,
            "BEAMGEO": self.beam_geo.to_dict() if self.beam_geo else None,
            "DKRF": self.dkrf.to_dict() if self.dkrf else None,
        }
        _dict.update(self.unmanaged_params)
        return _dict

    to_dict.__doc__ = _ReconsParam.to_dict.__doc__

    @staticmethod
    def from_dict(_dict):
        recons_params = ReconsParams()
        recons_params.load_from_dict(_dict)
        return recons_params

    from_dict.__doc__ = _ReconsParam.from_dict.__doc__

    def load_from_dict(self, _dict):
        self._load_unmanaged_params(_dict=_dict)
        if _dict["FT"] is None:
            self._ft = None
        else:
            self._ft.load_from_dict(_dict["FT"])
        if _dict["PYHSTEXE"] is None:
            self._pyhst = None
        else:
            self._pyhst.load_from_dict(_dict["PYHSTEXE"])
        if _dict["FTAXIS"] is None:
            self._axis = None
        else:
            self._axis.load_from_dict(_dict["FTAXIS"])
        if _dict["PAGANIN"] is None:
            self._paganin = None
        else:
            self._paganin.load_from_dict(_dict["PAGANIN"])
        if _dict["BEAMGEO"] is None:
            self._beam_geo = None
        else:
            self._beam_geo.load_from_dict(_dict["BEAMGEO"])
        if _dict["DKRF"] is None:
            self._dkrf = None
        else:
            self._dkrf.load_from_dict(_dict["DKRF"])

    load_from_dict.__doc__ = _ReconsParam.load_from_dict.__doc__

    def __iter__(self):
        yield self.ft
        yield self.pyhst
        yield self.axis
        yield self.paganin
        yield self.beam_geo
        yield self.dkrf

    def copy(self, other_rp):
        """
        copy parameters value from other_rp

        :param _ReconsParam: reconsparam to copy
        :param Union[list, None] parameters: parameters names that we want to
                                             copy
        """
        if other_rp is None:
            return

        def copy_all_recons_params():
            """function called if other_rp is an instance of ReconsParams"""
            assert isinstance(other_rp, ReconsParams)
            other_sub_rps = (
                other_rp.ft,
                other_rp.pyhst,
                other_rp.axis,
                other_rp.paganin,
                other_rp.beam_geo,
                other_rp.dkrf,
            )
            for other_sub_rp in other_sub_rps:
                self.copy(other_sub_rp)
            self._set_unmanaged_params(other_rp.unmanaged_params)

        from tomwer.core.process.reconstruction.darkref.params import (
            DKRFRP,
        )  # avoid cyclic import. DKRF should be specific to dark and flat field and not in relation with ftserie

        if isinstance(other_rp, ReconsParams):
            copy_all_recons_params()
        elif isinstance(other_rp, FTRP):
            self._copy_ft_rp(other_rp)
        elif isinstance(other_rp, PyhstRP):
            self._copy_pyhst_rp(other_rp)
        elif isinstance(other_rp, AxisRP):
            self._copy_axis_rp(other_rp)
        elif isinstance(other_rp, PaganinRP):
            self._copy_paganin_rp(other_rp)
        elif isinstance(other_rp, BeamGeoRP):
            self._copy_beam_geo_rp(other_rp)
        elif isinstance(other_rp, DKRFRP):
            self._copy_dkrf_rp(other_rp)
        else:
            raise TypeError(
                "Cannot copy %s, because type %s is not recognized"
                "" % (other_rp, type(other_rp))
            )

    def _copy_ft_rp(self, other_rp):
        # note: we cannot pass by the deep copy function, fail with the
        # properties
        if self.ft is None:
            self.ft = other_rp.__class__()
        self.ft.copy(other_rp)

    def _copy_pyhst_rp(self, other_rp):
        if self.pyhst is None:
            self.pyhst = other_rp.__class__()
        self.pyhst.copy(other_rp)

    def _copy_axis_rp(self, other_rp):
        if self.axis is None:
            self.axis = other_rp.__class__()
        self.axis.copy(other_rp)

    def _copy_paganin_rp(self, other_rp):
        if self.paganin is None:
            self.paganin = other_rp.__class__()
        self.paganin.copy(other_rp)

    def _copy_beam_geo_rp(self, other_rp):
        if self.beam_geo is None:
            self.beam_geo = other_rp.__class__()
        self.beam_geo.copy(other_rp)

    def _copy_dkrf_rp(self, other_rp):
        if self.dkrf is None:
            self.dkrf = other_rp.__class__()
        self.dkrf.copy(other_rp)

    def _reset_unmanaged_params(self):
        _ReconsParam._reset_unmanaged_params(self)
        for sub_rp in self:
            sub_rp._reset_unmanaged_params()



# coding: utf-8
# /*##########################################################################
#
# Copyright (c) 2016-2017 European Synchrotron Radiation Facility
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# ###########################################################################*/

__authors__ = ["H. Payno"]
__license__ = "MIT"
__date__ = "05/07/2017"


import os
import shutil
import fnmatch

from tomwer.core.process.task import Task

from tomwer.core.scan.scanfactory import ScanFactory
from tomwer.core.settings import get_lbsram_path, get_dest_path
from tomwer.core.signal import Signal
from tomwer.core.utils import logconfig, rebaseParFile
import logging
from tomwer.core.scan.scanbase import TomwerScanBase
from tomwer.core.scan.edfscan import EDFTomoScan
from tomwer.core.scan.hdf5scan import HDF5TomoScan
from tomwer.io.utils import get_linked_files_with_entry
import tomwer.version

logger = logging.getLogger(__name__)

try:
    from tomwer.synctools.rsyncmanager import RSyncManager
except ImportError:
    logger.warning("rsyncmanager not available")
    has_rsync = False
else:
    has_rsync = True


class ScanTransfer(Task, input_names=("data",), output_names=("data",)):
    """Manage the copy of scan.

    .. warning : the destination directory is find out from the file system
                 if /lbsramxxx exists for example...
                 In the case we couldn't found the output directory then we
                 will ask for the user to set it.
    """

    scanready = Signal(TomwerScanBase)

    def __init__(self, varinfo=None, inputs=None, label=None):
        super().__init__(varinfo=varinfo, inputs=inputs, label=label)
        self.turn_off_print = inputs.get("turn_off_print", False)
        self._destDir = inputs.get("dest_dir", None)
        """
        output directory if forced. By default based by the env variable
        'TARGET_OUTPUT_FOLDER' if exists, else set to '/data/visitor'
         """
        self._copying = inputs.get("copying", False)
        force_sync = inputs.get("force_sync", None)
        if force_sync is not None:
            from silx.utils.deprecation import deprecated_warning

            deprecated_warning(
                type_="Parameter",
                name="force_sync",
                reason="Two parameters for the same option",
                replacement="block",
            )
        else:
            force_sync = False

        self._block = inputs.get("block", force_sync)

        self._move = inputs.get("move", False)
        if not isinstance(self._move, bool):
            raise TypeError("move is expected to be a boolean")

        self._force = inputs.get("force", False)
        if not isinstance(self._force, bool):
            raise TypeError("move is expected to be a boolean")

        self._noRsync = inputs.get("noRsync", False)  # TODO: rename noRsync to no_rsync
        if not isinstance(self._noRsync, bool):
            raise TypeError("move is expected to be a boolean")

    def set_configuration(self, properties):
        # No properties stored for now
        if "dest_dir" in properties:
            self.setDestDir(properties["dest_dir"])

    @staticmethod
    def program_name():
        """Name of the program used for this processing"""
        return "data transfer"

    @staticmethod
    def program_version():
        """version of the program used for this processing"""
        return tomwer.version.version

    @staticmethod
    def definition():
        """definition of the process"""
        return "transfer data from a folder to another"

    @staticmethod
    def getDefaultOutputDir():
        """Return the default output dir based on the computer setup"""
        if "TARGET_OUTPUT_FOLDER" in os.environ:
            return os.environ["TARGET_OUTPUT_FOLDER"]

        if os.path.isdir("/data/visitor"):
            return "/data/visitor"

        return ""

    def _process_edf_scan(self, scan, move=False, force=True, noRsync=False):
        if not isinstance(scan, EDFTomoScan):
            raise TypeError(f"{scan} is expected to be an instance of {EDFTomoScan}")
        outputdir = self.getDestinationDir(scan.path)
        if outputdir is None:
            return

        self._pretransfertOperations(scan.path, outputdir)
        # as we are in the workflow we want this function to be bloking.
        # so we will not used a thread for folder synchronization
        # for now rsync is not delaing with force option
        if not has_rsync or noRsync is True or RSyncManager().has_rsync() is False:
            logger.info("Can't use rsync, copying files")
            try:
                if move is True:
                    self._moveFiles(
                        scanPath=scan.path,
                        outputdir=os.path.dirname(outputdir),
                        force=force,
                    )
                else:
                    self._copyFiles(
                        scanPath=scan.path, outputdir=outputdir, force=force
                    )
            except shutil.Error as e:
                raise e
            else:
                output_scan = scan._deduce_transfert_scan(outputdir)
                try:
                    output_scan._update_latest_recons_urls(
                        old_path=scan.path, new_path=output_scan.path
                    )
                except Exception as e:
                    logger.warning(
                        "Fail to convert url of latest reconstruction. Reason is:" + e
                    )
                    output_scan.clear_latest_reconstructions()
                self.__noticeTransfertSuccess(input_scan=scan, output_scan=output_scan)
        else:
            source = scan.path
            if not source.endswith(os.path.sep):
                source = source + os.path.sep
            target = outputdir

            if not target.endswith(os.path.sep):
                target = target + os.path.sep

            self._signalCopying(scanID=source, outputdir=target)
            output_scan = scan._deduce_transfert_scan(outputdir)
            try:
                output_scan._update_latest_recons_urls(
                    old_path=scan.path, new_path=output_scan.path
                )
            except Exception as e:
                logger.warning(
                    "Fail to convert url of latest reconstruction. Reason is:" + e
                )
                output_scan.clear_latest_reconstructions()
            RSyncManager().sync_file(
                source=source,
                target=target,
                wait=self._block,
                delete=True,
                callback=self.__noticeTransfertSuccess,
                callback_parameters=(scan, output_scan),
                rights=777,
            )
        self.register_output(key="data", value=outputdir)
        return output_scan

    def _get_hdf5_dst_scan(self, bliss_scan_folder_path):
        if self._destDir is not None:
            rel_path = os.path.join(*bliss_scan_folder_path.split(os.sep)[-2:])
            return os.path.dirname(os.path.join(self._destDir, rel_path))
        # try to get outputdir from spec
        scanIDPath = os.path.abspath(bliss_scan_folder_path)
        return self._getOutputDirSpec() or os.path.join(
            *self._getOutputDirLBS(scanIDPath).split(os.sep)[:-1]
        )

    def _get_hdf5_sample_file_or_nx_dst(self, bliss_sample_file):
        bliss_sample_file = os.path.realpath(bliss_sample_file)
        if self._destDir is not None:
            rel_path = os.path.join(*bliss_sample_file.split(os.sep)[-2:])
            return os.path.join(self._destDir, rel_path)
        # try to get outputdir from spec
        bliss_sample_file = os.path.abspath(bliss_sample_file)
        return self._getOutputDirSpec() or self._getOutputDirLBS(bliss_sample_file)

    def _get_master_sample_file_dst(self, master_sample_file):
        if self._destDir is not None:
            rel_path = os.path.join(*master_sample_file.split(os.sep)[-2:])
            return os.path.join(self._destDir, rel_path)
        # try to get outputdir from spec
        master_sample_file = os.path.abspath(master_sample_file)
        return self._getOutputDirSpec() or self._getOutputDirLBS(master_sample_file)

    def _get_hdf5_proposal_file_dst(self, bliss_proposal_file):
        if self._destDir is not None:
            return os.path.join(self._destDir, os.path.split(bliss_proposal_file)[-1])
        # try to get outputdir from spec
        bliss_sample_file = os.path.abspath(bliss_proposal_file)
        return self._getOutputDirSpec() or self._getOutputDirLBS(bliss_sample_file)

    def _process_hdf5_scan(self, scan) -> TomwerScanBase:
        assert isinstance(scan, HDF5TomoScan)
        from tomwer.core.process.control.datalistener import DataListener

        files_sources = []
        files_dest = []
        delete_opt = []

        if os.path.exists(scan.process_file):
            source_scans = DataListener.get_source_scans(
                process_file=scan.process_file, entry=scan.entry
            )
        else:
            source_scans = None
        # manage scan folder
        if source_scans is not None:
            for source_scan in source_scans:
                # files_sources.append(source_scan)
                # files_dest.append(
                #    self._get_hdf5_dst_scan(bliss_scan_folder_path=source_scan)
                # )
                # delete_opt.append(True)
                # scan folder is no more managed. Now this is done by browsing VDS directly
                pass

        # manage proposal file
        if os.path.exists(scan.process_file):
            proposal_file = DataListener.get_proposal_file(
                process_file=scan.process_file, entry=scan.entry
            )
        else:
            proposal_file = None
        if proposal_file is not None:
            files_sources.append(proposal_file)
            files_dest.append(self._get_hdf5_proposal_file_dst(proposal_file))
            delete_opt.append(False)

        # manage sample file
        if os.path.exists(scan.process_file):
            sample_file = DataListener.get_sample_file(
                process_file=scan.process_file, entry=scan.entry
            )
        else:
            sample_file = None
        if sample_file is not None:
            files_sources.append(sample_file)
            files_dest.append(self._get_hdf5_sample_file_or_nx_dst(sample_file))
            delete_opt.append(False)

        # manage saving file
        if os.path.exists(scan.process_file):
            mst_sample_file = DataListener.get_master_sample_file(
                process_file=scan.process_file, entry=scan.entry
            )
        else:
            mst_sample_file = None
        if mst_sample_file is not None:
            files_sources.append(mst_sample_file)
            files_dest.append(self._get_master_sample_file_dst(mst_sample_file))
            delete_opt.append(False)

        # manage .nx file
        if os.path.exists(scan.master_file):
            (
                new_nx_file,
                files_sources_for_nx,
                files_dest_for_nx,
            ) = self.handle_nexus_file(scan.master_file, scan.entry)
            files_sources.extend(files_sources_for_nx)
            files_dest.extend(files_dest_for_nx)
            delete_opt.append([True] * len(files_dest_for_nx))
            files_sources.append(scan.master_file)
            files_dest.append(new_nx_file)
            delete_opt.append(True)
            output_scan = HDF5TomoScan(scan=new_nx_file, entry=scan.entry)
        else:
            output_scan = None

        # manage files generated (*slice*.h5/edf, *.cfg, *.par...)
        # for reconstructed file, .h5, .edf if there is some conflict at one
        # point I guess we might need to check file entry ? or rename the file
        # according to the entry.

        # manage .par, .cfg and .rec files if any
        patterns = ["*.par", "*.cfg", "*.rec", "*.log"]
        # manage *slice*.hdf5 and *slice*.edf files (reconstructed slice)
        patterns += [
            "*slice*.hdf5",
            "*slice*.h5",
            "*slice*.jpeg",
            "*slice*.jpg",
            "*slice*.tiff",
            "*slice*.tif",
            "*slice*.j2k",
        ]
        # manage *vol files
        patterns += [
            "*_vol",
            "*_vol.hdf5",
        ]
        # manage nabu and tomwer processes files
        patterns += [
            "*tomwer_processes.h5",
            "*nabu_processes.hdf5",
            "steps_file_basename_nabu_sinogram_save_step.hdf5",
        ]
        # manage new dark and flat files
        patterns += [
            "dark.hdf5",
            "darks.hdf5",
            "flats.hdf5",
        ]

        def match_pattern(file_name):
            file_name = file_name.lower()
            for pattern in patterns:
                if fnmatch.fnmatch(file_name, pattern):
                    return True
            return False

        dir_name = os.path.dirname(scan.master_file)
        for file_ in os.listdir(dir_name):
            if match_pattern(file_name=file_):
                full_file_path = os.path.join(dir_name, file_)
                files_sources.append(full_file_path)
                files_dest.append(self._get_hdf5_sample_file_or_nx_dst(full_file_path))
                delete_opt.append(True)

        # manage folders
        patterns = [
            "nabu_cfg_files",
            "*slice*",
            "saaxis_results",
            "sadeltabeta_results",
            "steps_file_basename_nabu_sinogram_save_step",
        ]
        for folder_ in os.listdir(dir_name):
            if match_pattern(file_name=folder_):
                full_file_path = os.path.join(dir_name, folder_)
                files_sources.append(full_file_path)
                # don't know why we need this os.sep ? call another get dest ?
                # os.path.join(os.sep, self._get_hdf5_dst_scan(full_file_path), folder_)
                dest = os.path.join(os.sep, self._get_hdf5_dst_scan(full_file_path))
                files_dest.append(dest)
                delete_opt.append(True)

        RSyncManager().sync_files(
            sources=files_sources,
            targets=files_dest,
            wait=self._block,
            delete=delete_opt,
            callback=self.__noticeTransfertSuccess,
            callback_parameters=(scan, output_scan),
        )
        return output_scan

    def handle_nexus_file(self, src_nexus_file: str, entry: str) -> tuple:
        """
        Return a tuple of three elemts:
        src_nexus_file, files_sources_for_nx, files_dest_for_nx

        :param str src_nexus_file: source file to treat
        :return: (dst_nexus_file, files_sources_for_nx, files_dest_for_nx)
                * dst_nexus_file: str -> output nexus file that will be created by the transfer
                * list files_sources_for_nx: list of relative files connected to source nexus file
                * files_dest_for_nx: list of creating files from files_sources_for_nx once transfer is accomplish
        :rtype: tuple
        """
        dst_nx_file = self._get_hdf5_sample_file_or_nx_dst(src_nexus_file)
        linked_datasets_src = get_linked_files_with_entry(
            hdf5_file=src_nexus_file, entry=entry
        )
        linked_files_src = list(map(lambda x: x[0], linked_datasets_src))
        relative_linked_files_src = list(
            filter(lambda file_path: not os.path.isabs(file_path), linked_files_src)
        )
        relative_linked_files_dst = [
            os.path.normpath(os.path.join(os.path.dirname(dst_nx_file), file_path))
            for file_path in relative_linked_files_src
        ]
        return dst_nx_file, relative_linked_files_src, relative_linked_files_dst

    def run(self):
        """Launch the process process

        :param scan: the path to the file we want to move/process
        :type scan: :class:`.TomoBase`
        :param move: if True, directly move the files. Otherwise copy the files
        :param force: if True then force the copy even if the file/folder already
            exists
        :param bool noRSync: True if we wan't do sue shutil instead of rsync.
        """
        scan = self.inputs.data
        if scan is None:
            raise ValueError("'scan' should be provided")

        if scan is None:
            self.outputs.data = None
        force = self._force

        _scan = scan
        if type(_scan) is dict:
            _scan = ScanFactory.create_scan_object_frm_dict(scan)

        assert isinstance(_scan, TomwerScanBase)

        logger.info("synchronisation with scanPath")
        if isinstance(scan, EDFTomoScan):
            output_scan = self._process_edf_scan(
                scan=scan, move=self._move, force=force, noRsync=self._noRsync
            )
        elif isinstance(scan, HDF5TomoScan):
            if self._move is True:
                raise NotImplementedError("move option not implemented")
            if self._noRsync is True:
                raise NotImplementedError("noRsync option not implemented")
            output_scan = self._process_hdf5_scan(scan=scan)
        else:
            raise TypeError("Other scan than EDF or HDF5 are not managed")
        if self._return_dict:
            self.outputs.data = output_scan.to_dict()
        else:
            self.outputs.data = output_scan

    def _pretransfertOperations(self, scanfolder, outputdir):
        """Operation to be run before making the transfert of the scan"""
        self._updateParFiles(scanfolder, outputdir)

    def _updateParFiles(self, scanfolder, outputdir):
        """Update all path contained in the .par files to fit the new outpudir"""
        if not os.path.exists(scanfolder):
            return
        for _file in os.listdir(scanfolder):
            if _file.lower().endswith(".par"):
                rebaseParFile(
                    os.path.join(scanfolder, _file),
                    oldfolder=scanfolder,
                    newfolder=outputdir,
                )

    def __noticeTransfertSuccess(self, input_scan, output_scan):
        self._signalCopySucceed()

        results = {"input_scan": str(input_scan), "output_scan": str(output_scan)}
        entry = "entry"
        if isinstance(output_scan, HDF5TomoScan):
            entry = output_scan.entry
        self.register_process(
            process_file=output_scan.process_file,
            entry=entry,
            results=results,
            process_index=output_scan.pop_process_index(),
            configuration=None,
        )
        logger.processSucceed(
            "transfer succeed of {} to {}".format(str(input_scan), str(output_scan)),
            extra={
                logconfig.DOC_TITLE: self._scheme_title,
                logconfig.FROM: str(input_scan),
                logconfig.TO: str(output_scan),
            },
        )
        self.signalTransfertOk(input_scan=input_scan, output_scan=output_scan)

    def signalTransfertOk(self, input_scan, output_scan):
        if input_scan is None or output_scan is None:
            return
        assert isinstance(input_scan, TomwerScanBase)
        assert isinstance(output_scan, TomwerScanBase)
        self.scanready.emit(output_scan)

    def _copyFiles(self, scanPath, outputdir, force):
        """Copying files and removing them"""
        assert type(scanPath) is str
        assert type(outputdir) is str
        assert os.path.isdir(scanPath)
        #        if force is False:
        #            assert(os.path.isdir(outputdir))
        # create the destination dir
        if not os.path.isdir(outputdir):

            def createDirAndTopDir(_dir):
                if not os.path.isdir(os.path.dirname(_dir)):
                    createDirAndTopDir(os.path.dirname(_dir))
                os.makedirs(_dir)

            createDirAndTopDir(outputdir)
        # we can't copy directly the top folder because he is already existing
        for f in os.listdir(scanPath):
            file = os.path.join(scanPath, f)
            fileDest = os.path.join(outputdir, f)
            if force is True:
                if os.path.isdir(fileDest):
                    shutil.rmtree(fileDest)
                if os.path.isfile(fileDest):
                    os.remove(fileDest)
            if os.path.exists(fileDest):
                raise FileExistsError(fileDest, "already exists")
            if os.path.isdir(file):
                shutil.copytree(src=file, dst=fileDest)
            else:
                shutil.copy2(src=file, dst=fileDest)

        info = "Removing directory at %s" % scanPath
        logger.info(info)
        shutil.rmtree(scanPath)
        info = "sucessfuly removed file at %s !!!" % scanPath
        logger.info(info)

    def _moveFiles(self, scanPath, outputdir, force):
        """Function simply moving files"""
        assert os.path.isdir(scanPath)
        if force is False:
            assert os.path.isdir(outputdir)

        logger.debug(
            "synchronisation with scanPath",
            extra={logconfig.DOC_TITLE: self._scheme_title},
        )

        target = os.path.join(outputdir, os.path.basename(scanPath))
        if force is True and os.path.isdir(target):
            shutil.rmtree(target)
        shutil.move(scanPath, outputdir)

    def _requestFolder(self):
        out = None
        while out is None:
            out = input("please give the output directory : \n")
            if not os.path.isdir(out):
                warning = "given path " + out
                warning += " is not a directory, please give a valid directory"
                logger.warning(warning)
                out = None
        return out

    def _getOutputDirSpec(self):
        return None

    def _getOutputDirLBS(self, scanPath):
        if scanPath.startswith(get_lbsram_path()):
            return scanPath.replace(get_lbsram_path(), get_dest_path(), 1)
        else:
            return None

    def getDestinationDir(self, scanPath):
        """Return the destination directory. The destination directory is the
        root directory"""
        if self._destDir is not None:
            return os.path.join(self._destDir, os.path.basename(scanPath))

        # try to get outputdir from spec
        scanIDPath = os.path.abspath(scanPath)

        outputdir = self._getOutputDirSpec() or self._getOutputDirLBS(scanIDPath)
        if outputdir is None:
            outputdir = self._requestFolder()

        return outputdir

    def setDestDir(self, dist):
        """Force the outpudir to dist.

        :param str dist: path to the folder. If None remove force behavior
        """
        self._destDir = dist
        if self._destDir is not None and os.path.isdir(self._destDir):
            logger.warning("Given path %s is not a directory" % self._destDir)

    # some function to print the output in the terminal #

    def _signalCopying(self, scanID, outputdir):
        self._copying = True
        if self.turn_off_print is False:
            print("######################################")
            print("###")
            print("###")
            print("### copying files ", scanID, " to ", outputdir)
            print("### ...")

        info = "start moving folder from %s to %s" % (scanID, outputdir)
        logger.processStarted(info, extra={logconfig.DOC_TITLE: self._scheme_title})

    def _signalCopyFailed(self):
        self._copying = False
        if self.turn_off_print is False:
            print("###")
            print("### copy failed")
            print("###")
            print("######################################")

    def _signalCopySucceed(self):
        self._copying = False
        if self.turn_off_print is False:
            print("###")
            print("### copy succeeded")
            print("###")
            print("######################################")

    def isCopying(self):
        """

        :return: True if the folder transfert is actually doing a copy
        """
        return self._copying

    def setForceSync(self, b):
        """

        :param bool b: if True then folderTransfert will wait until transfert
            is done to be released. Otherwise will launch a 'free' thread wich
            will notice transfert end later.
        """
        self._block = b

    def getOutput(self, scan):
        """

        :param scan:
        :return:
        """
        return os.path.join(self.getDestinationDir(scan), os.path.basename(scan))



# coding: utf-8
# /*##########################################################################
#
# Copyright (c) 2016-2017 European Synchrotron Radiation Facility
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# ###########################################################################*/

__authors__ = ["H. Payno"]
__license__ = "MIT"
__date__ = "22/20/2020"


from orangewidget.widget import OWBaseWidget
from processview.core.superviseprocess import SuperviseProcess
import functools
from processview.core.manager import ProcessManager, DatasetState


class SuperviseOW(OWBaseWidget, SuperviseProcess, openclass=True):
    """
    A basic OWWidget but registered on the process manager
    """

    def __init__(self, parent, process_id=None):
        OWBaseWidget.__init__(self, parent, process_id=process_id)
        SuperviseProcess.__init__(self, process_id=process_id)
        self.destroyed.connect(functools.partial(ProcessManager().unregister, self))

    def setCaption(self, caption):
        OWBaseWidget.setCaption(self, caption)
        self.name = caption

    def notify_skip(self, scan, details=None):
        ProcessManager().notify_dataset_state(
            dataset=scan, process=self, state=DatasetState.SKIPPED, details=details
        )

    def notify_pending(self, scan, details=None):
        ProcessManager().notify_dataset_state(
            dataset=scan, process=self, state=DatasetState.PENDING, details=details
        )

    def notify_succeed(self, scan, details=None):
        ProcessManager().notify_dataset_state(
            dataset=scan, process=self, state=DatasetState.SUCCEED, details=details
        )

    def notify_failed(self, scan, details=None):
        ProcessManager().notify_dataset_state(
            dataset=scan, process=self, state=DatasetState.FAILED, details=details
        )



# coding: utf-8
# /*##########################################################################
#
# Copyright (c) 2016-2017 European Synchrotron Radiation Facility
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# ###########################################################################*/

__authors__ = ["v. Valls", "H. Payno"]
__license__ = "MIT"
__date__ = "26/02/2019"

from silx.gui import qt
from silx.gui.plot import CompareImages as _CompareImages
from silx.gui import icons as silx_icons
from tomwer.gui import icons as tomwer_icons
from silx.gui.plot import tools
import numpy
import weakref
import logging

_logger = logging.getLogger(__name__)


class CompareImagesToolBar(qt.QToolBar):
    """ToolBar containing specific tools to custom the configuration of a
    :class:`CompareImages` widget

    Use :meth:`setCompareWidget` to connect this toolbar to a specific
    :class:`CompareImages` widget.

    :param Union[qt.QWidget,None] parent: Parent of this widget.
    """

    def __init__(self, parent=None):
        qt.QToolBar.__init__(self, parent)

        self.__compareWidget = None

        menu = qt.QMenu(self)
        self.__visualizationAction = qt.QAction(self)
        self.__visualizationAction.setMenu(menu)
        self.__visualizationAction.setCheckable(False)
        self.addAction(self.__visualizationAction)
        self.__visualizationGroup = qt.QActionGroup(self)
        self.__visualizationGroup.setExclusive(True)
        self.__visualizationGroup.triggered.connect(self.__visualizationModeChanged)

        icon = silx_icons.getQIcon("compare-mode-a")
        action = qt.QAction(icon, "Display frame 1", self)
        action.setIconVisibleInMenu(True)
        action.setCheckable(True)
        action.setShortcut(qt.QKeySequence(qt.Qt.Key_A))
        action.setProperty("mode", CompareImages.VisualizationMode.ONLY_A)
        menu.addAction(action)
        self.__aModeAction = action
        self.__visualizationGroup.addAction(action)

        icon = silx_icons.getQIcon("compare-mode-b")
        action = qt.QAction(icon, "Display frame 2", self)
        action.setIconVisibleInMenu(True)
        action.setCheckable(True)
        action.setShortcut(qt.QKeySequence(qt.Qt.Key_B))
        action.setProperty("mode", CompareImages.VisualizationMode.ONLY_B)
        menu.addAction(action)
        self.__bModeAction = action
        self.__visualizationGroup.addAction(action)

        icon = silx_icons.getQIcon("compare-mode-rbneg-channel")
        action = qt.QAction(icon, "Subtract in color mode", self)
        action.setIconVisibleInMenu(True)
        action.setCheckable(True)
        action.setShortcut(qt.QKeySequence(qt.Qt.Key_C))
        action.setProperty(
            "mode", _CompareImages.VisualizationMode.COMPOSITE_RED_BLUE_GRAY_NEG
        )
        menu.addAction(action)
        self.__ycChannelModeAction = action
        self.__visualizationGroup.addAction(action)

        icon = tomwer_icons.getQIcon("compare_mode_a_minus_b")
        action = qt.QAction(icon, "Subtract in BW mode", self)
        action.setIconVisibleInMenu(True)
        action.setCheckable(True)
        action.setShortcut(qt.QKeySequence(qt.Qt.Key_S))
        action.setProperty("mode", _CompareImages.VisualizationMode.COMPOSITE_A_MINUS_B)
        menu.addAction(action)
        self.__ycChannelModeAction = action
        self.__visualizationGroup.addAction(action)

    def setCompareWidget(self, widget):
        """
        Connect this tool bar to a specific :class:`CompareImages` widget.

        :param Union[None,CompareImages] widget: The widget to connect with.
        """
        compareWidget = self.getCompareWidget()
        if compareWidget is not None:
            compareWidget.sigConfigurationChanged.disconnect(
                self.__updateSelectedActions
            )
        compareWidget = widget
        if compareWidget is None:
            self.__compareWidget = None
        else:
            self.__compareWidget = weakref.ref(compareWidget)
        if compareWidget is not None:
            widget.sigConfigurationChanged.connect(self.__updateSelectedActions)
        self.__updateSelectedActions()

    def getCompareWidget(self):
        """Returns the connected widget.

        :rtype: CompareImages
        """
        if self.__compareWidget is None:
            return None
        else:
            return self.__compareWidget()

    def __updateSelectedActions(self):
        """
        Update the state of this tool bar according to the state of the
        connected :class:`CompareImages` widget.
        """
        widget = self.getCompareWidget()
        if widget is None:
            return

        mode = widget.getVisualizationMode()
        action = None
        for a in self.__visualizationGroup.actions():
            actionMode = a.property("mode")
            if mode == actionMode:
                action = a
                break
        old = self.__visualizationGroup.blockSignals(True)
        if action is not None:
            # Check this action
            action.setChecked(True)
        else:
            action = self.__visualizationGroup.checkedAction()
            if action is not None:
                # Uncheck this action
                action.setChecked(False)
        self.__updateVisualizationMenu()
        self.__visualizationGroup.blockSignals(old)

    def __visualizationModeChanged(self, selectedAction):
        """Called when user requesting changes of the visualization mode."""
        self.__updateVisualizationMenu()
        widget = self.getCompareWidget()
        if widget is not None:
            mode = selectedAction.property("mode")
            widget.setVisualizationMode(mode)

    def __updateVisualizationMenu(self):
        """Update the state of the action containing visualization menu."""
        selectedAction = self.__visualizationGroup.checkedAction()
        if selectedAction is not None:
            self.__visualizationAction.setText(selectedAction.text())
            self.__visualizationAction.setIcon(selectedAction.icon())
            self.__visualizationAction.setToolTip(selectedAction.toolTip())
        else:
            self.__visualizationAction.setText("")
            self.__visualizationAction.setIcon(qt.QIcon())
            self.__visualizationAction.setToolTip("")

    def __alignmentModeChanged(self, selectedAction):
        """Called when user requesting changes of the alignment mode."""
        self.__updateAlignmentMenu()
        widget = self.getCompareWidget()
        if widget is not None:
            mode = selectedAction.property("mode")
            widget.setAlignmentMode(mode)

    def __updateAlignmentMenu(self):
        """Update the state of the action containing alignment menu."""
        selectedAction = self.__alignmentGroup.checkedAction()
        if selectedAction is not None:
            self.__alignmentAction.setText(selectedAction.text())
            self.__alignmentAction.setIcon(selectedAction.icon())
            self.__alignmentAction.setToolTip(selectedAction.toolTip())
        else:
            self.__alignmentAction.setText("")
            self.__alignmentAction.setIcon(qt.QIcon())
            self.__alignmentAction.setToolTip("")

    def __keypointVisibilityChanged(self):
        """Called when action managing keypoints visibility changes"""
        widget = self.getCompareWidget()
        if widget is not None:
            keypointsVisible = self.__displayKeypoints.isChecked()
            widget.setKeypointsVisible(keypointsVisible)


class CompareImages(_CompareImages.CompareImages):
    def _createToolBars(self, plot):
        """Create tool bars displayed by the widget"""
        self._interactiveModeToolBar = tools.InteractiveModeToolBar(
            parent=self, plot=plot
        )
        self._imageToolBar = tools.ImageToolBar(parent=self, plot=plot)

        toolBar = CompareImagesToolBar(self)
        toolBar.setCompareWidget(self)
        self._compareToolBar = toolBar

    def _createStatusBar(self, plot):
        self._statusBar = CompareImagesStatusBar(self)
        self._statusBar.setCompareWidget(self)

    def getCompareToolBar(self):
        return self._compareToolBar


class CompareImagesStatusBar(qt.QStatusBar):
    """StatusBar containing specific information contained in a
    :class:`CompareImages` widget

    Use :meth:`setCompareWidget` to connect this toolbar to a specific
    :class:`CompareImages` widget.

    :param Union[qt.QWidget,None] parent: Parent of this widget.
    """

    def __init__(self, parent=None):
        qt.QStatusBar.__init__(self, parent)
        self.setSizeGripEnabled(False)
        self.layout().setSpacing(0)
        self.__compareWidget = None
        self._label1 = qt.QLabel(self)
        self._label1.setFrameShape(qt.QFrame.WinPanel)
        self._label1.setFrameShadow(qt.QFrame.Sunken)
        self._label2 = qt.QLabel(self)
        self._label2.setFrameShape(qt.QFrame.WinPanel)
        self._label2.setFrameShadow(qt.QFrame.Sunken)
        self._transform = qt.QLabel(self)
        self._transform.setFrameShape(qt.QFrame.WinPanel)
        self._transform.setFrameShadow(qt.QFrame.Sunken)
        self.addWidget(self._label1)
        self.addWidget(self._label2)
        self.addWidget(self._transform)
        self._pos = None
        self._updateStatusBar()

    def setCompareWidget(self, widget):
        """
        Connect this tool bar to a specific :class:`CompareImages` widget.

        :param Union[None,CompareImages] widget: The widget to connect with.
        """
        compareWidget = self.getCompareWidget()
        if compareWidget is not None:
            compareWidget.getPlot().sigPlotSignal.disconnect(self.__plotSignalReceived)
            compareWidget.sigConfigurationChanged.disconnect(self.__dataChanged)
        compareWidget = widget
        if compareWidget is None:
            self.__compareWidget = None
        else:
            self.__compareWidget = weakref.ref(compareWidget)
        if compareWidget is not None:
            compareWidget.getPlot().sigPlotSignal.connect(self.__plotSignalReceived)
            compareWidget.sigConfigurationChanged.connect(self.__dataChanged)

    def getCompareWidget(self):
        """Returns the connected widget.

        :rtype: CompareImages
        """
        if self.__compareWidget is None:
            return None
        else:
            return self.__compareWidget()

    def __plotSignalReceived(self, event):
        """Called when old style signals at emmited from the plot."""
        if event["event"] == "mouseMoved":
            x, y = event["x"], event["y"]
            self.__mouseMoved(x, y)

    def __mouseMoved(self, x, y):
        """Called when mouse move over the plot."""
        self._pos = x, y
        self._updateStatusBar()

    def __dataChanged(self):
        """Called when internal data from the connected widget changes."""
        self._updateStatusBar()

    def _formatData(self, data):
        """Format pixel of an image.

        It supports intensity, RGB, and RGBA.

        :param Union[int,float,numpy.ndarray,str]: Value of a pixel
        :rtype: str
        """
        if data is None:
            return "-"
        if isinstance(data, (int, numpy.integer)):
            return "%d" % data
        if isinstance(data, (float, numpy.floating)):
            return "%f" % data
        if isinstance(data, numpy.ndarray):
            # RGBA value
            if data.shape == (3,):
                return "R:%d G:%d B:%d" % (data[0], data[1], data[2])
            elif data.shape == (4,):
                return "R:%d G:%d B:%d A:%d" % (data[0], data[1], data[2], data[3])
        _logger.debug("Unsupported data format %s. Cast it to string.", type(data))
        return str(data)

    def _updateStatusBar(self):
        """Update the content of the status bar"""
        widget = self.getCompareWidget()
        if widget is None:
            self._label1.setText("Frame 1: -")
            self._label2.setText("Frame 2: -")
            self._transform.setVisible(False)
        else:
            transform = widget.getTransformation()
            self._transform.setVisible(transform is not None)
            if transform is not None:
                has_notable_translation = not numpy.isclose(
                    transform.tx, 0.0, atol=0.01
                ) or not numpy.isclose(transform.ty, 0.0, atol=0.01)
                has_notable_scale = not numpy.isclose(
                    transform.sx, 1.0, atol=0.01
                ) or not numpy.isclose(transform.sy, 1.0, atol=0.01)
                has_notable_rotation = not numpy.isclose(transform.rot, 0.0, atol=0.01)

                strings = []
                if has_notable_translation:
                    strings.append("Translation")
                if has_notable_scale:
                    strings.append("Scale")
                if has_notable_rotation:
                    strings.append("Rotation")
                if strings == []:
                    has_translation = not numpy.isclose(
                        transform.tx, 0.0
                    ) or not numpy.isclose(transform.ty, 0.0)
                    has_scale = not numpy.isclose(
                        transform.sx, 1.0
                    ) or not numpy.isclose(transform.sy, 1.0)
                    has_rotation = not numpy.isclose(transform.rot, 0.0)
                    if has_translation or has_scale or has_rotation:
                        text = "No big changes"
                    else:
                        text = "No changes"
                else:
                    text = "+".join(strings)
                self._transform.setText("Align: " + text)

                strings = []
                if not numpy.isclose(transform.ty, 0.0):
                    strings.append("Translation x: %0.3fpx" % transform.tx)
                if not numpy.isclose(transform.ty, 0.0):
                    strings.append("Translation y: %0.3fpx" % transform.ty)
                if not numpy.isclose(transform.sx, 1.0):
                    strings.append("Scale x: %0.3f" % transform.sx)
                if not numpy.isclose(transform.sy, 1.0):
                    strings.append("Scale y: %0.3f" % transform.sy)
                if not numpy.isclose(transform.rot, 0.0):
                    strings.append(
                        "Rotation: %0.3fdeg" % (transform.rot * 180 / numpy.pi)
                    )
                if strings == []:
                    text = "No transformation"
                else:
                    text = "\n".join(strings)
                self._transform.setToolTip(text)

            if self._pos is None:
                self._label1.setText("Frame 1: -")
                self._label2.setText("Frame 2: -")
            else:
                data1, data2 = widget.getRawPixelData(self._pos[0], self._pos[1])
                if isinstance(data1, str):
                    self._label1.setToolTip(data1)
                    text1 = "-"
                else:
                    self._label1.setToolTip("")
                    text1 = self._formatData(data1)
                if isinstance(data2, str):
                    self._label2.setToolTip(data2)
                    text2 = "-"
                else:
                    self._label2.setToolTip("")
                    text2 = self._formatData(data2)
                self._label1.setText("Frame 1: %s" % text1)
                self._label2.setText("Frame 2: %s" % text2)



# coding: utf-8
# /*##########################################################################
#
# Copyright (c) 2016-2017 European Synchrotron Radiation Facility
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# ###########################################################################*/

__authors__ = ["C. Nemoz", "H. Payno"]
__license__ = "MIT"
__date__ = "10/01/2018"


import logging
from ..utils import WidgetLongProcessing
from silx.gui import qt
from orangewidget import gui
from orangecontrib.tomwer.orange.managedprocess import SuperviseOW
from orangewidget.settings import Setting
from orangewidget.widget import Input, Output
from tomwer.core.scan.scanbase import TomwerScanBase
from tomwer.gui.reconstruction.darkref.darkrefcopywidget import DarkRefAndCopyWidget
from tomwer.synctools.ftseries import QReconsParams
from tomwer.web.client import OWClient
from tomwer.synctools.stacks.reconstruction.dkrefcopy import DarkRefCopyProcessStack
from processview.core.manager import DatasetState
from processview.core.manager import ProcessManager
from tomwer.utils import docstring
import tomwer.core.process.reconstruction.darkref.darkrefscopy
import copy

_logger = logging.getLogger(__name__)


class DarkRefAndCopyOW(SuperviseOW, OWClient, WidgetLongProcessing):
    """
    A simple widget managing the copy of an incoming folder to an other one

    :param parent: the parent widget
    """

    # note of this widget should be the one registered on the documentation
    name = "dark and flat field construction"
    id = "orange.widgets.tomwer.darkrefs"
    description = "This widget will generate dark refs for a received scan "
    icon = "icons/darkref.svg"
    priority = 15
    keywords = ["tomography", "dark", "darks", "ref", "refs", "flat", "flats"]

    want_main_area = True
    resizing_enabled = True
    compress_signal = False

    _rpSetting = Setting(dict())
    """Setting to load and save DarkRefAndCopyWidget settings"""
    # this one is keep for backward compatibility. Will be removed on 0.10 I guess

    static_input = Setting({"data": None, "dark_ref_params": None})

    sigScanReady = qt.Signal(TomwerScanBase)
    """Signal emitted when a scan is ready"""

    ewokstaskclass = (
        tomwer.core.process.reconstruction.darkref.darkrefscopy.DarkRefsCopy
    )

    class Inputs:
        data = Input(name="data", type=TomwerScanBase, doc="one scan to be process")

    class Outputs:
        data = Output(name="data", type=TomwerScanBase, doc="one scan to be process")

    def __init__(self, parent=None, reconsparams=None):
        """

        :param QReconsParams reconsparams: reconstruction parameters
        """
        SuperviseOW.__init__(self, parent)
        OWClient.__init__(self)
        WidgetLongProcessing.__init__(self)
        recons_params = reconsparams or QReconsParams()
        self._processing_stack = DarkRefCopyProcessStack(process_id=self.process_id)

        dark_ref_params = self.static_input.get("dark_ref_params", None)
        if dark_ref_params is None:
            dark_ref_params = self._rpSetting
        if dark_ref_params != dict():
            try:
                recons_params.dkrf.load_from_dict(dark_ref_params)
            except Exception:
                _logger.warning("fail to load reconstruction settings")

        self.widget = DarkRefAndCopyWidget(
            parent=self, reconsparams=recons_params, process_id=self.process_id
        )
        self._layout = gui.vBox(self.mainArea, self.name).layout()
        self._layout.addWidget(self.widget)
        self.setForceSync = self.widget.setForceSync

        # expose API
        self.hasRefStored = self.widget.hasFlatStored
        self.setModeAuto = self.widget.set_mode_auto
        self.setRefsFromScan = self.widget.setRefsFromScan
        self.setCopyActive = self.widget.setCopyActive
        self.hasDarkStored = self.widget.hasDarkStored
        self.hasFlatStored = self.widget.hasFlatStored

        # connect signal / slot
        self.widget.sigProcessingStart.connect(self._startProcessing)
        self.widget.sigProcessingEnd.connect(self._endProcessing)
        self.widget.sigScanReady.connect(self.signalReady)
        self.widget.recons_params.sigChanged.connect(self._updateSettingsVals)
        self.widget.sigModeAutoChanged.connect(self._updateSettingsVals)
        self.widget.sigCopyActivationChanged.connect(self._updateSettingsVals)
        self._processing_stack.sigComputationStarted.connect(self._startProcessing)
        self._processing_stack.sigComputationEnded.connect(self._endProcessing)

        # load some other copy parameters
        if dark_ref_params != dict():
            try:
                if "activate" in dark_ref_params:
                    self.widget.setCopyActive(dark_ref_params.pop("activate"))
                if "auto" in dark_ref_params:
                    auto_mode = dark_ref_params.pop("auto")
                    # insure backward compatibility. Has beem saved as a tuple (this was a typo)
                    if isinstance(auto_mode, tuple) and isinstance(auto_mode[0], bool):
                        auto_mode = auto_mode[0]
                    self.widget.setModeAuto(auto_mode)
            except Exception:
                _logger.warning("fail to load reconstruction settings")

        self.setCaption(self.windowTitle())

    @Inputs.data
    def process(self, scanID):
        if scanID is None:
            return
        assert isinstance(scanID, TomwerScanBase)
        ProcessManager().notify_dataset_state(
            dataset=scanID, process=self, state=DatasetState.PENDING
        )
        self._processing_stack.add(
            copy.copy(scanID), configuration=self.widget.recons_params.to_dict()
        )

    @docstring(SuperviseOW)
    def reprocess(self, dataset):
        self.process(dataset)

    def signalReady(self, scanID):
        assert isinstance(scanID, TomwerScanBase)
        self.Outputs.data.send(scanID)
        self.sigScanReady.emit(scanID)

    def _updateSettingsVals(self):
        self._rpSetting = self.widget.recons_params.to_dict()
        self._rpSetting["auto"] = self.widget.isOnModeAuto()
        self._rpSetting["activate"] = self.widget.isCopyActive()
        self.static_input = {
            "data": None,
            "dark_ref_params": self.widget.recons_params.to_dict(),
        }
        self.static_input["dark_ref_params"]["auto"] = self.widget.isOnModeAuto()
        self.static_input["dark_ref_params"]["activate"] = self.widget.isCopyActive()

    @property
    def recons_params(self):
        return self.widget.recons_params

    def close(self):
        self.widget.close()
        super(DarkRefAndCopyOW, self).close()

    def _endProcessing(self, scan):
        WidgetLongProcessing._endProcessing(self, scan)
        self.Outputs.data.send(scan)
        self.sigScanReady.emit(scan)
        _logger.info("{} ended".format(str(scan)))



# coding: utf-8
###########################################################################
# Copyright (C) 2016-2019 European Synchrotron Radiation Facility
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
#############################################################################

__authors__ = ["H.Payno"]
__license__ = "MIT"
__date__ = "08/06/2021"


from processview.core.manager import ProcessManager
from tomwer.synctools.sadeltabeta import QSADeltaBetaParams
from tomwer.synctools.axis import QAxisRP
from tomwer.core.scan.scanbase import TomwerScanBase
from tomwer.core.process.reconstruction.sadeltabeta.sadeltabeta import (
    SADeltaBetaProcess,
)
from tomwer.core.settings import get_lbsram_path, isOnLbsram
from tomwer.core.utils import isLowOnMemory
from processview.core.manager import DatasetState
from ..processingstack import FIFO, ProcessingThread
from processview.core.superviseprocess import SuperviseProcess
from tomwer.io.utils import format_stderr_stdout
from silx.gui import qt
import logging
import functools

_logger = logging.getLogger(__name__)


class SADeltaBetaProcessStack(FIFO, qt.QObject):
    """Implementation of the `.AxisProcess` but having a stack for treating
    scans and making computation in threads"""

    def __init__(self, sa_delta_beta_params, process_id=None):
        qt.QObject.__init__(self)
        FIFO.__init__(self, process_id=process_id)
        assert sa_delta_beta_params is not None
        self._dry_run = False
        self._process_fct = None

    def patch_processing(self, process_fct):
        self._computationThread.patch_processing(process_fct)

    def set_dry_run(self, dry_run):
        self._dry_run = dry_run

    def _process(self, scan, configuration, callback=None):
        ProcessManager().notify_dataset_state(
            dataset=scan,
            process=self,
            state=DatasetState.ON_GOING,
        )
        _logger.processStarted("start sa-delta-beta {}" "".format(str(scan)))
        assert isinstance(scan, TomwerScanBase)
        if scan.axis_params is None:
            scan.axis_params = QAxisRP()
        if scan.sa_delta_beta_params is None:
            scan.sa_delta_beta_params = QSADeltaBetaParams()
        self._scan_currently_computed = scan
        sa_delta_beta_params = QSADeltaBetaParams.from_dict(configuration)
        if isOnLbsram(scan) and isLowOnMemory(get_lbsram_path()) is True:
            # if computer is running into low memory on lbsram skip it
            mess = "low memory, skip sa-delta-beta-axis calculation", scan.path
            ProcessManager().notify_dataset_state(
                dataset=scan, process=self._process_id, state=DatasetState.SKIPPED
            )
            _logger.processSkipped(mess)
            scan.sa_delta_beta_params.set_value(None)
            if callback is not None:
                callback()
            self.scan_ready(scan=scan)
        else:
            self._scan_currently_computed = scan
            self._computationThread.init(
                scan=scan, sa_delta_beta_params=sa_delta_beta_params
            )
            # need to manage connect before starting it because
            fct_callback = functools.partial(self._end_threaded_computation, callback)
            self._computationThread.finished.connect(fct_callback)
            self._computationThread.start()

    def _end_computation(self, scan, future_scan, callback):
        """
        callback when the computation thread is finished

        :param scan: pass if no call to '_computationThread is made'
        """
        assert isinstance(scan, TomwerScanBase)
        FIFO._end_computation(
            self, scan=scan, future_scan=future_scan, callback=callback
        )

    def _end_threaded_computation(self, callback=None):
        assert self._scan_currently_computed is not None
        self._computationThread.finished.disconnect()
        if callback:
            callback()
        FIFO._end_threaded_computation(self)

    def _create_processing_thread(self, process_id=None) -> qt.QThread:
        return _ProcessingThread(process_id=process_id)


class _ProcessingThread(ProcessingThread, SuperviseProcess):
    """
    Thread use to execute the processing of the axis position
    """

    def __init__(self, process_id=None):
        SuperviseProcess.__init__(self, process_id=process_id)
        try:
            ProcessingThread.__init__(self, process_id=process_id)
        except TypeError:
            ProcessingThread.__init__(self)
        self.center_of_rotation = None
        self._dry_run = False
        self._scan = None
        self._sa_delta_beta_params = None
        self._patch_process_fct = None
        """function pointer to know which function to call for the axis
        calculation"""
        self.__patch = {}
        """Used to patch some calculation method (for test purpose)"""

    def set_dry_run(self, dry_run):
        self._dry_run = dry_run

    def patch_processing(self, process_fct):
        self._patch_process_fct = process_fct

    def init(self, scan, sa_delta_beta_params):
        self._scan = scan
        self._sa_delta_beta_params = sa_delta_beta_params

    def run(self):
        self.sigComputationStarted.emit()
        if self._patch_process_fct:
            scores = {}
            for db in self._sa_delta_beta_params.delta_beta_values:
                scores[db] = self._patch_process_fct(db)
            self._scan.sa_delta_beta_params.scores = scores
            SADeltaBetaProcess.autofocus(scan=self._scan)
            self.db_value = self._scan.sa_delta_beta_params.autofocus
        else:
            sa_delta_beta = SADeltaBetaProcess(
                inputs={
                    "dump_process": False,
                    "data": self._scan,
                    "dry_run": self._dry_run,
                    "sa_delta_beta_params": self._sa_delta_beta_params,
                },
                process_id=self.process_id,
            )
            try:
                sa_delta_beta.run()
            except Exception as e:
                _logger.error(str(e))
                mess = "SADeltaBeta computation for {} failed.".format(str(self._scan))
                state = DatasetState.FAILED
            else:
                mess = "SADeltaBeta computation for {} succeed.".format(str(self._scan))
                state = DatasetState.WAIT_USER_VALIDATION
                self.db_value = self._scan.sa_delta_beta_params.autofocus

            nabu_logs = []
            for std_err, std_out in zip(sa_delta_beta.std_errs, sa_delta_beta.std_outs):
                nabu_logs.append(format_stderr_stdout(stdout=std_out, stderr=std_err))
            self._nabu_log = nabu_logs
            nabu_logs.insert(0, mess)
            details = "\n".join(nabu_logs)
            ProcessManager().notify_dataset_state(
                dataset=self._scan,
                process=self,
                state=state,
                details=details,
            )



# coding: utf-8
# /*##########################################################################
#
# Copyright (c) 2017 European Synchrotron Radiation Facility
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# ###########################################################################*/

__authors__ = ["H. Payno"]
__license__ = "MIT"
__date__ = "22/06/2018"


import fabio
import os
import logging

logger = logging.getLogger(__name__)


def getMotorInformationFromEDfFile(_file):
    """

    :param _file: file to the edf file containing header with information
    :return: dictionary with motor name as key and motor position as value
    :rtype: dict
    """
    assert type(_file) is str
    assert _file.endswith(".edf")
    if not os.path.isfile(_file):
        warn = "%s is not a valid file. Cannot retrieve motor information" % _file
        logger.warning(warn)
        return None
    else:
        with fabio.open(_file) as dsc:
            header = dsc.header

        if "motor_mne" not in header:
            logger.warning(
                'key "motor_mne" not found in the file header.'
                "Cannot retrieve motor information"
            )
            return None

        if "motor_pos" not in header:
            logger.warning(
                'key "motor_pos" not found in the file header.'
                "Cannot retrieve motor information"
            )
            return None

        motor_mne_lst = header["motor_mne"].split(" ")
        motor_pos_lst = header["motor_pos"].split(" ")
        if len(motor_mne_lst) != len(motor_pos_lst):
            logger.warning(
                "Incoherent number of motor_mne and motor_pos."
                "Cannot retrieve motor information"
            )
            return None
        res = {}
        for mne, pos in zip(motor_mne_lst, motor_pos_lst):
            res[mne] = pos
        return res



# /*##########################################################################
# Copyright (C) 2016-2017 European Synchrotron Radiation Facility
#
# This file is part of the PyMca X-ray Fluorescence Toolkit developed at
# the ESRF by the Software group.
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
#############################################################################*/

__author__ = ["H. Payno"]
__license__ = "MIT"
__date__ = "10/01/2018"


import logging
from silx.gui import qt
from tomwer.core.process.reconstruction.ftseries.params.paganin import PaganinMode
from tomwer.core.utils.char import BETA_CHAR, DELTA_CHAR
from tomwer.gui.reconstruction.ftserie.h5editor import H5StructEditor
from tomwer.gui.utils.inputwidget import SelectionLineEdit
from tomwer.synctools.ftseries import _QPaganinRP, QReconsParams

logger = logging.getLogger(__name__)


class PaganinWidget(H5StructEditor, qt.QWidget):
    """
    Definition of the tab enabling Paganin reconstruction parameters edition

    :param reconsparams: reconstruction parameters edited by the widget
    """

    def __init__(self, reconsparams, parent=None):
        qt.QWidget.__init__(self, parent=parent)
        H5StructEditor.__init__(self, structID="PAGANIN")
        self._recons_params = None
        self.setReconsParams(recons_params=reconsparams)

        self._groupHideIfNotMulti = []
        # list of widget to hide if the multi mode is not activated
        self._groupHideIfOff = []
        # list of widget to hide if Paganin is off

        self.setLayout(qt.QVBoxLayout())

        self.__buildMode()
        self.__buildUnsharp()
        self.__buildThreshold()
        self.__buildDilate()
        self.__buildMedianMedianFilterSize()
        self.__buidMKeep()

        self.__updatePaganinMode(0)

        spacer = qt.QWidget(self)
        spacer.setSizePolicy(qt.QSizePolicy.Minimum, qt.QSizePolicy.Expanding)
        self.layout().addWidget(spacer)

        self._makeConnection()

    def setReconsParams(self, recons_params):
        if isinstance(recons_params, QReconsParams):
            _recons_params = recons_params.paganin
        elif isinstance(recons_params, _QPaganinRP):
            _recons_params = recons_params
        else:
            raise ValueError(
                "recons_params should be an instance of QReconsParam or _QPaganinRP"
            )

        if self._recons_params:
            self._recons_params.sigChanged.disconnect(self._update_params)
        self._recons_params = _recons_params
        self.load(self._recons_params)
        self._recons_params.sigChanged.connect(self._update_params)

    def _update_params(self):
        """Update all parameter"""
        self.load(self._recons_params)

    def clear(self):
        pass

    def _makeConnection(self):
        self._qcbpaganin.currentIndexChanged.connect(self._modeChanged)
        self._qleSigmaBeta.editingFinished.connect(self._DBChanged)
        self._qleSigmaBeta2.editingFinished.connect(self._DB2Changed)
        self._unsharp_sigma_coeff.editingFinished.connect(self._unsharpCoeffChanged)
        self._unsharp_sigma_mask_value.editingFinished.connect(
            self._unsharpSigmachanged
        )
        self._qleThreshold.editingFinished.connect(self._thresholdChanged)
        self._qleDilatation.editingFinished.connect(self._dilateChanged)
        self._qleMedianFilterSize.editingFinished.connect(self._medianRChanged)
        self._qcbKeepBone.toggled.connect(self._keepBoneChanged)
        self._qcbKeepSoft.toggled.connect(self._keepSoftChanged)
        self._qcbKeepAbs.toggled.connect(self._keepAbsChanged)
        self._qcbKeepCorr.toggled.connect(self._keepCorrChanged)
        self._qcbKeepMask.toggled.connect(self._keepMaskChanged)

    def __buildMode(self):
        widget = qt.QWidget(self)
        widget.setLayout(qt.QGridLayout())
        # define paganin combo box
        self._qcbpaganin = qt.QComboBox(self)
        self._qcbpaganin.setSizePolicy(qt.QSizePolicy.Expanding, qt.QSizePolicy.Minimum)
        # TODO: use the _paganinRP
        for mode in PaganinMode:
            self._qcbpaganin.addItem(mode.name)
        widget.layout().addWidget(qt.QLabel("Mode", parent=widget), 0, 0)
        widget.layout().addWidget(self._qcbpaganin, 0, 1)
        self._qcbpaganin.currentIndexChanged.connect(self.__updatePaganinMode)
        self.linkComboboxWithH5Variable(self._qcbpaganin, "MODE", fitwithindex=True)

        self._buildAlphaBetaWidgets(widget)
        self.layout().addWidget(widget)
        self._groupHideIfOff.append(self.paganinModeOpt)

    def _buildAlphaBetaWidgets(self, widget):
        self.paganinModeOpt = qt.QWidget(widget)
        self.paganinModeOpt.setLayout(qt.QGridLayout())
        label = DELTA_CHAR + " / " + BETA_CHAR
        self.paganinModeOpt.layout().addWidget(qt.QLabel(label, parent=widget), 1, 0)
        self._qleSigmaBeta = qt.QLineEdit("0", self)
        self.paganinModeOpt.layout().addWidget(self._qleSigmaBeta, 1, 1)
        self.LinkLineEditWithH5Variable(self._qleSigmaBeta, "DB", float)

        label_multi = DELTA_CHAR + " / " + BETA_CHAR + " (multi)"
        lMulti = qt.QLabel(label_multi, parent=widget)
        self.paganinModeOpt.layout().addWidget(lMulti)
        self._qleSigmaBeta2 = qt.QLineEdit("0", self)
        self.paganinModeOpt.layout().addWidget(self._qleSigmaBeta2, 2, 1)
        self.LinkLineEditWithH5Variable(self._qleSigmaBeta2, "DB2", float)
        self._groupHideIfNotMulti.append(lMulti)
        self._groupHideIfNotMulti.append(self._qleSigmaBeta2)

        widget.layout().addWidget(self.paganinModeOpt, 1, 1)

    def _modeChanged(self):
        value = self._qcbpaganin.currentIndex()
        assert isinstance(self._recons_params, _QPaganinRP)
        self._recons_params["MODE"] = value

    def _DBChanged(self):
        value = float(self._qleSigmaBeta.text())
        self._recons_params["DB"] = value

    def _DB2Changed(self):
        value = float(self._qleSigmaBeta2.text())
        self._recons_params["DB2"] = value

    def __buildUnsharp(self):
        # unsharp label
        self._unsharp_group = qt.QGroupBox(title="unsharp mask parameters", parent=self)
        self.layout().addWidget(self._unsharp_group)
        self._unsharp_group.setLayout(qt.QGridLayout())
        label = "mask " + DELTA_CHAR + " value in pixels"
        self._unsharp_group.layout().addWidget(qt.QLabel(label), 0, 0)
        self._unsharp_sigma_coeff = qt.QLineEdit("0", self._unsharp_group)
        self._unsharp_group.layout().addWidget(self._unsharp_sigma_coeff, 0, 1)
        self.LinkLineEditWithH5Variable(
            self._unsharp_sigma_coeff, "UNSHARP_COEFF", float
        )

        self._unsharp_group.layout().addWidget(qt.QLabel("coefficient "), 1, 0)
        self._unsharp_sigma_mask_value = qt.QLineEdit("0", self._unsharp_group)
        validator = qt.QDoubleValidator(parent=self._unsharp_sigma_mask_value)
        self._unsharp_sigma_mask_value.setValidator(validator)
        self._unsharp_group.layout().addWidget(self._unsharp_sigma_mask_value, 1, 1)
        self.LinkLineEditWithH5Variable(
            self._unsharp_sigma_mask_value, "UNSHARP_SIGMA", float
        )

        self.layout().addWidget(self._unsharp_group)
        self._groupHideIfOff.append(self._unsharp_group)

    def _unsharpCoeffChanged(self):
        value = float(self._unsharp_sigma_coeff.text())
        self._recons_params["UNSHARP_COEFF"] = value

    def _unsharpSigmachanged(self):
        value = float(self._unsharp_sigma_mask_value.text())
        self._recons_params["UNSHARP_SIGMA"] = value

    def __buildThreshold(self):
        widget = qt.QWidget(self)
        widget.setLayout(qt.QHBoxLayout())
        widget.layout().addWidget(
            qt.QLabel("Threshold for high absorption mask", parent=widget)
        )
        self._qleThreshold = qt.QLineEdit("0", widget)
        widget.layout().addWidget(self._qleThreshold)
        self.LinkLineEditWithH5Variable(self._qleThreshold, "THRESHOLD", float)

        self.layout().addWidget(widget)
        self._groupHideIfNotMulti.append(widget)
        self._groupHideIfOff.append(widget)

    def _thresholdChanged(self):
        value = float(self._qleThreshold.text())
        self._recons_params["THRESHOLD"] = value

    def __buildDilate(self):
        widget = qt.QWidget(self)
        widget.setLayout(qt.QHBoxLayout())
        widget.layout().addWidget(
            qt.QLabel("Dilatation to cover the dark fringes", parent=widget)
        )
        self._qleDilatation = qt.QLineEdit("0", parent=widget)
        widget.layout().addWidget(self._qleDilatation)
        self.LinkLineEditWithH5Variable(self._qleDilatation, "DILATE", int)

        self.layout().addWidget(widget)
        self._groupHideIfNotMulti.append(widget)
        self._groupHideIfOff.append(widget)

    def _dilateChanged(self):
        value = int(self._qleDilatation.text())
        self._recons_params["DILATE"] = value

    def __buildMedianMedianFilterSize(self):
        widget = qt.QWidget(self)
        widget.setLayout(qt.QHBoxLayout())
        widget.layout().addWidget(qt.QLabel("Median filter size", parent=widget))
        self._qleMedianFilterSize = qt.QLineEdit("", parent=widget)
        widget.layout().addWidget(self._qleMedianFilterSize)
        self.LinkLineEditWithH5Variable(self._qleMedianFilterSize, "MEDIANR", int)

        self.layout().addWidget(widget)
        self._groupHideIfNotMulti.append(widget)
        self._groupHideIfOff.append(widget)

    def _medianRChanged(self):
        value = int(self._qleMedianFilterSize.text())
        self._recons_params["MEDIANR"] = value

    def __buidMKeep(self):
        widget = qt.QWidget(self)
        widget.setLayout(qt.QVBoxLayout())

        self._qcbKeepBone = qt.QCheckBox(
            "Keep a separate volume for high absorption part", parent=widget
        )
        widget.layout().addWidget(self._qcbKeepBone)
        self.linkCheckboxWithH5Variable(self._qcbKeepBone, "MKEEP_BONE")

        self._qcbKeepSoft = qt.QCheckBox(
            "Keep a separate volume for low absorption part", parent=widget
        )
        widget.layout().addWidget(self._qcbKeepSoft)
        self.linkCheckboxWithH5Variable(self._qcbKeepSoft, "MKEEP_SOFT")

        self._qcbKeepAbs = qt.QCheckBox(
            "Keep a separate volume for absorption reconstruction", parent=widget
        )
        widget.layout().addWidget(self._qcbKeepAbs)
        self.linkCheckboxWithH5Variable(self._qcbKeepAbs, "MKEEP_ABS")

        self._qcbKeepCorr = qt.QCheckBox(
            "Keep binary mask (bone absorption - average neighbours)", parent=widget
        )
        widget.layout().addWidget(self._qcbKeepCorr)
        self.linkCheckboxWithH5Variable(self._qcbKeepCorr, "MKEEP_CORR")

        self._qcbKeepMask = qt.QCheckBox("Keep the binary mask", parent=widget)
        widget.layout().addWidget(self._qcbKeepMask)
        self.linkCheckboxWithH5Variable(self._qcbKeepMask, "MKEEP_MASK")

        self.layout().addWidget(widget)
        self._groupHideIfNotMulti.append(widget)
        self._groupHideIfOff.append(widget)

    def _keepBoneChanged(self, b):
        self._recons_params["MKEEP_BONE"] = b

    def _keepSoftChanged(self, b):
        self._recons_params["MKEEP_SOFT"] = b

    def _keepAbsChanged(self, b):
        self._recons_params["MKEEP_ABS"] = b

    def _keepCorrChanged(self, b):
        self._recons_params["MKEEP_CORR"] = b

    def _keepMaskChanged(self, b):
        self._recons_params["MKEEP_MASK"] = b

    def getPaganinMode(self):
        return self._qcbpaganin.currentText()

    def __updatePaganinMode(self, newindex: int):
        # Deal with PAGANIN node visibility
        [widget.setVisible(newindex != 0) for widget in self._groupHideIfOff]
        [widget.setVisible(newindex == 3) for widget in self._groupHideIfNotMulti]


class PaganinRangeWidget(PaganinWidget):
    def _buildAlphaBetaWidgets(self, widget):
        self.paganinModeOpt = qt.QWidget(widget)
        self.paganinModeOpt.setLayout(qt.QGridLayout())
        label = DELTA_CHAR + " / " + BETA_CHAR
        self.paganinModeOpt.layout().addWidget(qt.QLabel(label, parent=widget), 1, 0)
        self._qleSigmaBeta = SelectionLineEdit(text="0", parent=self)
        self.paganinModeOpt.layout().addWidget(self._qleSigmaBeta, 1, 1)
        self.LinkSelectionLineEditWithH5Variable(self._qleSigmaBeta, "DB", str)

        label_multi = DELTA_CHAR + " / " + BETA_CHAR + " (multi)"
        lMulti = qt.QLabel(label_multi, parent=widget)
        self.paganinModeOpt.layout().addWidget(lMulti)
        self._qleSigmaBeta2 = SelectionLineEdit(text="0", parent=self)
        self.paganinModeOpt.layout().addWidget(self._qleSigmaBeta2, 2, 1)
        self.LinkSelectionLineEditWithH5Variable(self._qleSigmaBeta2, "DB2", str)
        self._groupHideIfNotMulti.append(lMulti)
        self._groupHideIfNotMulti.append(self._qleSigmaBeta2)

        widget.layout().addWidget(self.paganinModeOpt, 1, 1)

    def _DBChanged(self):
        value = self._qleSigmaBeta.selection
        self._recons_params["DB"] = value

    def _DB2Changed(self):
        value = self._qleSigmaBeta2.selection
        self._recons_params["DB2"] = value



# coding: utf-8
# /*##########################################################################
#
# Copyright (c) 2017 European Synchrotron Radiation Facility
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# ###########################################################################*/
__authors__ = ["H. Payno"]
__license__ = "MIT"
__date__ = "05/11/2018"


from tomwer.gui.control.datalist import GenericDataListDialog
from tomwer.test.utils import skip_gui_test
from silx.gui.utils.testutils import TestCaseQt
from tomwer.core.utils.scanutils import MockEDF
from silx.gui import qt
import pytest
import tempfile


@pytest.mark.skipif(skip_gui_test(), reason="skip gui test")
class DataListTest(TestCaseQt):
    """Test that the datalist widget work correctly"""

    def setUp(self):
        super().setUp()
        self.widget = GenericDataListDialog(parent=None)
        self.widget._callbackRemoveAllFolders()
        self.widget.clear()
        self._folders = []
        for iFolder in range(5):
            self._folders.append(tempfile.mkdtemp())
            MockEDF.mockScan(
                scanID=self._folders[-1], nRadio=5, nRecons=5, nPagRecons=0, dim=10
            )

    def tearDown(self):
        self.widget.setAttribute(qt.Qt.WA_DeleteOnClose)
        self.widget.close()
        self.widget = None
        self._folders = None
        super().tearDown()

    def test(self):
        """simple test adding and removing folders"""
        for _folder in self._folders:
            self.widget.add(_folder)
        self.assertEqual(self.widget.length(), len(self._folders))
        self.widget.remove(self._folders[0])
        self.assertEqual(self.widget.length(), len(self._folders) - 1)
        self.assertTrue(self._folders[0] not in self.widget.datalist._scanIDs)
        self.widget.selectAll()
        self.widget._callbackRemoveFolder()
        self.assertEqual(self.widget.length(), 0)



# coding: utf-8
# /*##########################################################################
# Copyright (C) 2016 European Synchrotron Radiation Facility
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
#############################################################################*/
"""GUI to call RecPyHST
"""

__authors__ = ["H. Payno"]
__license__ = "MIT"
__date__ = "22/01/2017"

import logging
from silx.gui import qt
from tomwer.core.utils import pyhstutils
from tomwer.gui.reconstruction.ftserie.h5editor import H5StructsEditor
from tomwer.synctools.ftseries import QReconsParams, _QPyhstRP

logger = logging.getLogger(__name__)


class RecPyHSTWidget(H5StructsEditor, qt.QWidget):
    """
    Widget to interface the `makeRecPyHST` function
    """

    def __init__(self, recons_params, parent=None):
        qt.QWidget.__init__(self, parent)
        self.recons_params = None

        H5StructsEditor.__init__(self, structsManaged=("PYHSTEXE",))
        self.setLayout(qt.QVBoxLayout())
        self.layout().addWidget(self._buildPyHSTEXE())
        self.layout().addWidget(self._buildMakeOARFile())
        self.setReconsParams(recons_params)

    def setReconsParams(self, recons_params):
        if isinstance(recons_params, _QPyhstRP):
            _recons_params = recons_params
        elif isinstance(QReconsParams):
            _recons_params = recons_params.pyhst
        else:
            raise TypeError(
                "recons_params should be an instance ot PyhstRP or " "ReconsParams"
            )
        if self.recons_params:
            self.recons_params.sigChanged.disconnect(self._update)
        self.recons_params = _recons_params
        self.load(self.recons_params)
        self.recons_params.sigChanged.connect(self._update)

    def _buildPyHSTEXE(self):
        widget = qt.QWidget(self)
        widget.setLayout(qt.QVBoxLayout())

        exe_widget = qt.QWidget()
        widget.layout().addWidget(exe_widget)
        exe_widget.setLayout(qt.QHBoxLayout())

        # create the PyHST version combobox
        exe_widget.layout().addWidget(qt.QLabel("PyHST version : ", parent=exe_widget))
        self._qcbPyHSTVersion = qt.QComboBox(parent=self)
        self.linkComboboxWithH5Variable(
            self._qcbPyHSTVersion, structID="PYHSTEXE", h5ParamName="EXE"
        )
        exe_widget.layout().addWidget(self._qcbPyHSTVersion)

        # try to get the PyHST dir
        d = pyhstutils._getPyHSTDir()
        if d is None:
            logger.warning(
                """Can't find the directory containing the PyHST
                directory. Please set the environment variable
                PYHST_DIR and run again"""
            )
            d = "not found"
        else:
            availablePyHSTVersion = pyhstutils._findPyHSTVersions(d)
            availablePyHSTVersion.extend(pyhstutils._getPyHSTFromEnvVariable())
            if len(availablePyHSTVersion) == 0:
                logger.warning("No valid PyHST version found.")
            else:
                [self._qcbPyHSTVersion.addItem(exe) for exe in availablePyHSTVersion]

        pyhst_dir_widget = qt.QWidget(parent=widget)
        widget.layout().addWidget(pyhst_dir_widget)
        pyhst_dir_widget.setLayout(qt.QHBoxLayout())
        pyhst_dir_widget.layout().addWidget(qt.QLabel("PyHST directory:"))
        pyhst_dir_widget.layout().addWidget(qt.QLabel(d))

        self._qcbPyHSTVersion.currentIndexChanged.connect(self._pyHSTVersionChanged)

        return widget

    def _pyHSTVersionChanged(self):
        value = self._qcbPyHSTVersion.currentText()
        self.recons_params["EXE"] = value

    def _buildMakeOARFile(self):
        self._makeOARFileCB = qt.QCheckBox("make OAR file", parent=self)
        self.linkCheckboxWithH5Variable(
            self._makeOARFileCB, "PYHSTEXE", "MAKE_OAR_FILE", invert=False
        )
        self._makeOARFileCB.toggled.connect(self._makeOARChanged)
        return self._makeOARFileCB

    def _makeOARChanged(self, b):
        self.recons_params["MAKE_OAR_FILE"] = b

    def _update(self):
        self.load(self.recons_params)



# coding: utf-8
# /*##########################################################################
#
# Copyright (c) 2017 European Synchrotron Radiation Facility
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# ###########################################################################*/

__authors__ = ["H. Payno"]
__license__ = "MIT"
__date__ = "20/01/2017"

import logging
import operator
import os
import shutil
from silx.io.octaveh5 import Octaveh5

logger = logging.getLogger(__name__)


def get_vol_file_shape(file_path):
    ddict = {}
    f = open(file_path, "r")
    lines = f.readlines()
    for line in lines:
        if "=" not in line:
            continue
        line_str = line.rstrip().replace(" ", "")
        line_str = line_str.split("#")[0]
        key, value = line_str.split("=")
        ddict[key.lower()] = value

    dimX = None
    dimY = None
    dimZ = None

    if "num_z" in ddict:
        dimZ = int(ddict["num_z"])
    if "num_y" in ddict:
        dimY = int(ddict["num_y"])
    if "num_x" in ddict:
        dimX = int(ddict["num_x"])

    return (dimZ, dimY, dimX)


def orderFileByLastLastModification(folder, fileList):
    """Return the list of files sorted by time of last modification.
    From the oldest to the newest modify"""
    res = {}
    for f in fileList:
        res[os.path.getmtime(os.path.join(folder, f))] = f

    return sorted(res.items(), key=operator.itemgetter(0))


def generateDefaultH5File(path):
    """Write a h5 file for reconstruction with the default parameters values"""
    from tomwer.core.process.reconstruction.ftseries.params.fastsetupdefineglobals import (
        FastSetupAll,
    )  # avoid cyclic import

    ft = FastSetupAll()
    ft.writeAll(path, 3.8)


def getOrCreateH5File(scanID):
    """Try to get the h5 file containing reconstruction parameters.
    If not existing create it from a set of rule.

    :param str scanID: path to the folder containing the reconstruction to run
    :param defaultOctave: If no file found then create one from our own default
        setting. This require a call to OctaveH5 and t define the targetted
        octave version
    """
    # TODO : create a setting file ?
    f = getThH5FilePath(scanID)
    if os.path.isfile(f):
        return f
    else:
        default = os.path.expanduser("~") + "/.octave/mytomodefaults.h5"
        if default and os.path.isfile(default):
            shutil.copyfile(default, f)
            assert os.path.isfile(f)
            return f
        else:
            generateDefaultH5File(f)
            assert os.path.isfile(f)
            return f


def getThH5FilePath(scanID):
    """Return the theoretical path for the h5 file used for reconstruction for
    the given folder (scan)

    :param str scanID: the path of the folder
    """
    octave_h5_param = "octave_FT_params.h5"
    assert os.path.isdir(scanID)
    return os.path.join(scanID, octave_h5_param)


def tryToFindH5File(folder, politic):
    """Return a file in the given folder if any, if more than one, follows
    defined politic.

    :param folder: the folder to explore
    :param str politic: return the newest or oldest file if more than one file
    """
    assert os.path.isdir(folder)
    assert type(politic) is str
    assert politic.lower() in ("newest", "oldest")

    result = None
    age = None
    for f in os.listdir(folder):
        # if this is the master file in a .h5 acquisition
        if f.endswith(".h5"):
            # skip master files
            if f == (os.path.basename(folder) + ".h5"):
                continue
            # skip silx process files
            if f.endswith("tomwer_processes.h5"):
                continue

            fPath = os.path.join(folder, f)
            if age is None:
                age = os.path.getmtime(fPath)
                result = fPath
            else:
                currentFileAge = os.path.getmtime(fPath)
                if currentFileAge > age and politic == "oldest":
                    age = currentFileAge
                    result = f
                if currentFileAge < age and politic == "newest":
                    age = currentFileAge
                    result = f

    return result


def saveH5File(structs, h5File, displayInfo=True):
    """Function to write the reconstruction parameters into the h5 file

    :param dict structs: the reconstruction parameters
    :param str h5File: the path to the file to create
    :param bool displayInfo: add information in the log ?
    """

    if not h5File.lower().endswith(".h5"):
        h5File = h5File + ".h5"

    # check that the file exists
    if displayInfo is True:
        mess = "try to save .h5 file ()%s ..." % h5File
        logger.info(mess)

    writer = Octaveh5(3.8)
    try:
        writer.open(h5File, "w")

        for structID in structs:
            if structs[structID] is not None:
                writer.write(structID, structs[structID])

    finally:
        writer.close()



# coding: utf-8
# /*##########################################################################
#
# Copyright (c) 2017 European Synchrotron Radiation Facility
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# ###########################################################################*/

__authors__ = ["H. Payno"]
__license__ = "MIT"
__date__ = "18/03/2020"


from silx.gui import qt
from silx.gui.utils.testutils import TestCaseQt
from tomwer.test.utils import skip_gui_test
from tomwer.gui.control.datalistener import ConfigurationWidget
from silx.gui.utils.testutils import SignalListener
import pytest


@pytest.mark.skipif(skip_gui_test(), reason="skip gui test")
class TestDataListenerConfiguration(TestCaseQt):
    """
    Simple test the interface for configuration is working
    """

    def setUp(self):
        TestCaseQt.setUp(self)
        self._configWidget = ConfigurationWidget(parent=None)
        self.sig_listener = SignalListener()
        self._configWidget.sigConfigurationChanged.connect(self.sig_listener)
        self._configWidget.setHost("localhost")

    def tearDown(self):
        self._configWidget.setAttribute(qt.Qt.WA_DeleteOnClose)
        self._configWidget.close()
        self.qapp.processEvents()
        self._configWidget = None
        TestCaseQt.tearDown(self)

    def testConfiguration(self):
        self.assertEqual(
            self._configWidget.getConfiguration(), {"host": "localhost", "port": 4000}
        )
        self._configWidget.setPort(0)
        self._configWidget.setHost("toto")
        self.assertEqual(
            self._configWidget.getConfiguration(), {"host": "toto", "port": 0}
        )

    def testInteraction(self):
        """make sure the sigConfigurationChanged is emitted only when the
        user validate the modifications. This is done because we need to
        stop the thread and launch a new one, we don't wan't to do this
        automatically"""
        self._configWidget.setPort(4000)
        self._configWidget.setHost("tata")
        self.assertEqual(self.sig_listener.callCount(), 0)
        self._configWidget.validate()
        self.qapp.processEvents()
        self.assertEqual(self.sig_listener.callCount(), 1)



# coding: utf-8
# /*##########################################################################
# Copyright (C) 2016 European Synchrotron Radiation Facility
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
#############################################################################*/

"""
This module is containing benchmarks for rsync
"""

__authors__ = ["H.Payno"]
__license__ = "MIT"
__date__ = "07/06/2017"


import os
import shutil
import tempfile
import time

from tomwer.synctools.rsyncmanager import RSyncManager


class BenchRSyncClass(object):
    ORI_FOLDER = "/home/payno/testRSync/data/"

    def __init__(self):
        """ """
        self.folder1Path = tempfile.mkdtemp()
        print(self.folder1Path)
        self.folder2Path = tempfile.mkdtemp()

        assert os.path.isdir(self.folder1Path)
        assert os.path.isdir(self.folder2Path)
        # assert (os.path.isdir(BenchRSyncClass.ORI_FOLDER))
        # copy from original to folder 1
        # shutil.copytree(src=BenchRSyncClass.ORI_FOLDER,
        #                dst=os.path.join(self.folder1Path, 'test'))

    def __del__(self):
        if os.path.isdir(self.folder1Path):
            shutil.rmtree(self.folder1Path)

        if os.path.isdir(self.folder2Path):
            shutil.rmtree(self.folder2Path)

    def testCopy(self, options, parallel=False, nbIter=3):

        startTime = time.time()

        for it in range(nbIter):
            # copy from folder 1 to folder 2
            RSyncManager().syncFolderRaw(
                source=self.folder1Path,
                target=self.folder2Path,
                options=options,
                parallel=parallel,
                block=True,
            )

            # copy from folder 2 to folder 1
            RSyncManager().syncFolderRaw(
                source=self.folder2Path,
                target=self.folder1Path,
                options=options,
                parallel=parallel,
                block=True,
            )

        # check data integrity ?
        return (time.time() - startTime) / nbIter

    def testCopyFolders(self, f1, f2, options, parallel=False, nbIter=1):

        fullTime = 0

        for it in range(nbIter):
            startTime = time.time()

            # copy from folder 1 to folder 2
            RSyncManager().syncFolderRaw(
                source=f1, target=f2, options=options, parallel=parallel, block=True
            )

            fullTime += time.time() - startTime

            # copy from folder 2 to folder 1
            RSyncManager().syncFolderRaw(
                source=f2, target=f1, options=options, parallel=parallel, block=True
            )

        return fullTime / nbIter

    def testCopyTree(self, parallel=True):

        startTime = time.time()

        # copy from original to folder 1
        shutil.copytree(
            src=self.folder1Path, dst=os.path.join(self.folder2Path, "test")
        )

        shutil.rmtree(os.path.join(self.folder1Path, "test"))
        # copy from original to folder 1
        shutil.copytree(
            src=self.folder2Path, dst=os.path.join(self.folder1Path, "test")
        )

        # check data integrity ?
        return time.time() - startTime


def benchemark_rsync():
    benchmark = BenchRSyncClass()

    options = []
    options.append(["-a"])
    options.append(["-r"])

    f1 = "/lbsram/data/visitor/mi1226/bm05/nemoz/test1_"
    f2 = "/data/visitor/mi1226/bm05/nemoz/test1_"

    print("----- not using parallel ------")

    for opt in options:
        print("options are : ")
        print(opt)
        print(benchmark.testCopyFolders(f1, f2, opt, parallel=False))

    #    print('----- using parallel ------')

    #    for opt in options:
    #        print('options are : ')
    #        print(opt)
    #        print(l.testCopyFolders(f1, f2, opt, parallel=True))

    print("----- default shutil ------")
    print(benchmark.testCopyTree())


if __name__ == "__main__":
    benchemark_rsync()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import argparse
import sys
import logging
import glob
from silx.io.url import DataUrl
from tomwer.core.process.reconstruction.nabu.castvolume import cast_volumes


_logger = logging.getLogger(__name__)


def extract_urls(file_patterns, input_data_path) -> tuple:
    res = []
    for file_pattern in file_patterns:
        print("look for file_pattern", {file_pattern})
        for file_ in glob.glob(file_pattern):
            print("file_", file_)
            if file_.lower().endswith((".tiff", ".tif")):
                res.append(DataUrl(file_path=file_, scheme="fabio"))
            else:
                _logger.warning(f"Unable to handle {file_}. Skip it")
    return tuple(res)


def main(argv):
    parser = argparse.ArgumentParser(description=__doc__)
    parser.add_argument(
        "files",
        help="Files or data url to be cast",
        nargs="+",
    )
    parser.add_argument(
        "--input-data-path",
        help="Optional input data path for hdf5 files",
        default=None,
    )
    parser.add_argument(
        "--output-data-type",
        help="data type to create",
        default="uint 16",
    )
    parser.add_argument(
        "--output-file-format",
        help="Output file format",
        default="tiff",
    )
    parser.add_argument(
        "output_file_path",
        help="file path or folder path to save the volume.",
    )
    # for now we only handle tiff as output
    # parser.add_argument(
    #     "--output-data-path",
    #     help="for hdf5 files you can provide a data path. Otherwise will store volume at the root level under volume",
    #     default="/volume",
    # )
    parser.add_argument(
        "--overwrite",
        dest="overwrite",
        action="store_true",
        default=False,
        help="Overwrite file or dataset if exists",
    )

    options = parser.parse_args(argv[1:])
    volume_urls = extract_urls(
        file_patterns=options.files, input_data_path=options.input_data_path
    )
    if len(volume_urls) == 0:
        _logger.warning("No input found")
        return

    cast_volumes(
        input_volume_urls=volume_urls,
        output_data_type=options.output_data_type,
        output_file_format=options.output_file_format,
        output_file_path=options.output_file_path,
        overwrite=options.overwrite,
    )


if __name__ == "__main__":
    main(sys.argv)



# coding: utf-8
# /*##########################################################################
#
# Copyright (c) 2016-2017 European Synchrotron Radiation Facility
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# ###########################################################################*/

__authors__ = ["C. Nemoz", "H. Payno"]
__license__ = "MIT"
__date__ = "18/02/2018"


from silx.gui import qt
from operator import itemgetter
from tomwer.core.scan.hdf5scan import HDF5TomoScan
from tomwer.core.scan.scanbase import TomwerScanBase


class ScanHistory(qt.QWidget):
    """Widget used to display the lastest discovered scans"""

    def __init__(self, parent):
        qt.QWidget.__init__(self, parent)

        self.setLayout(qt.QVBoxLayout())
        self.layout().addWidget(qt.QLabel(""))

        self.scanHistory = qt.QTableView(parent=parent)
        self.scanHistory.setSelectionBehavior(qt.QAbstractItemView.SelectRows)
        self.scanHistory.setModel(
            _FoundScanModel(
                parent=self.scanHistory, header=("time", "type", "scan ID"), mlist={}
            )
        )
        self.scanHistory.resizeColumnsToContents()
        self.scanHistory.setSortingEnabled(True)
        self.setSizePolicy(qt.QSizePolicy.Expanding, qt.QSizePolicy.Expanding)

        self.layout().addWidget(self.scanHistory)
        header = self.scanHistory.horizontalHeader()
        if qt.qVersion() < "5.0":
            setResizeMode = header.setResizeMode
        else:
            setResizeMode = header.setSectionResizeMode
        setResizeMode(0, qt.QHeaderView.Fixed)
        setResizeMode(1, qt.QHeaderView.Fixed)
        setResizeMode(2, qt.QHeaderView.Stretch)
        header.setStretchLastSection(True)

    def update(self, scans):
        self.scanHistory.setModel(
            _FoundScanModel(
                parent=self, header=("time", "type", "scan ID"), mlist=scans
            )
        )
        self.scanHistory.resizeColumnsToContents()


class _FoundScanModel(qt.QAbstractTableModel):
    """
    Model for :class:_ScanHistory

    :param mlist: list of tuple (scan, time stamp)
    """

    def __init__(self, parent, header, mlist, *args):
        qt.QAbstractTableModel.__init__(self, parent, *args)
        self.header = header
        self.myList = mlist

    def rowCount(self, parent=None):
        if self.myList is None:
            return 0
        else:
            return len(self.myList)

    def columnCount(self, parent=None):
        return 3

    def sort(self, col, order):
        self.layoutAboutToBeChanged.emit()
        if self.myList is None:
            return
        self.myList = sorted(list(self.myList), key=itemgetter(col))
        if order == qt.Qt.DescendingOrder:
            self.myList = list(reversed(sorted(list(self.myList), key=itemgetter(col))))

        self.layoutChanged.emit()

    def data(self, index, role):
        if not index.isValid():
            return None
        elif role != qt.Qt.DisplayRole:
            return None
        if index.column() == 0:
            return self.myList[index.row()][1].strftime("%a %m - %d - %Y   - %H:%M:%S")
        elif index.column() == 1:
            path = self.myList[index.row()][0]
            if isinstance(path, TomwerScanBase):
                return path.type
            elif HDF5TomoScan.directory_contains_scan(path) or "@" in path:
                return "hdf5"
            else:
                return "edf"
        elif index.column() == 2:
            return self.myList[index.row()][0]

    def headerData(self, col, orientation, role):
        if orientation == qt.Qt.Horizontal and role == qt.Qt.DisplayRole:
            return self.header[col]
        return None



# /*##########################################################################
# Copyright (C) 2016-2017 European Synchrotron Radiation Facility
#
# This file is part of the PyMca X-ray Fluorescence Toolkit developed at
# the ESRF by the Software group.
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
#############################################################################*/

__author__ = ["H. Payno"]
__license__ = "MIT"
__date__ = "10/01/2018"


import logging
from silx.gui import qt
from tomwer.gui.reconstruction.ftserie.h5editor import H5StructEditor
from tomwer.synctools.ftseries import QReconsParams, _QBeamGeoRP

logger = logging.getLogger(__name__)


class BeamGeoWidget(H5StructEditor, qt.QWidget):
    """
    Definition of the PyHST tab to edit the Geometry parameters

    :param reconsparams: reconstruction parameters edited by the widget
    """

    def __init__(self, parent=None, reconsparams=None):
        qt.QWidget.__init__(self, parent)
        H5StructEditor.__init__(self, structID="BEAMGEO")
        self._recons_params = None
        self.setReconsParams(recons_params=reconsparams)

        self.setLayout(qt.QVBoxLayout())
        self.layout().addWidget(self.__buildType())
        self.layout().addWidget(self.__buildSXSYDist())

        spacer = qt.QWidget(self)
        spacer.setSizePolicy(qt.QSizePolicy.Minimum, qt.QSizePolicy.Expanding)
        self.layout().addWidget(spacer)

        # set up
        self._geometryChanged()
        # connect signal / slot
        self._makeConnection()

    def setReconsParams(self, recons_params):
        if isinstance(recons_params, QReconsParams):
            _recons_params = recons_params.beam_geo
        elif isinstance(recons_params, _QBeamGeoRP):
            _recons_params = recons_params
        else:
            raise ValueError(
                "recons_params should be an instance of QReconsParam or _QBeamGeoRP"
            )

        if self._recons_params:
            self._recons_params.sigChanged.disconnect(self._update_params)
        self._recons_params = _recons_params
        self.load(self._recons_params)
        self._recons_params.sigChanged.connect(self._update_params)

    def _update_params(self):
        """Update all parameter"""
        self.load(self._recons_params)

    def _makeConnection(self):
        self._qcbType.currentIndexChanged.connect(self._typeChanged)
        self._qleSX.editingFinished.connect(self._SXChanged)
        self._qleSY.editingFinished.connect(self._SYChanged)
        self._qleDIST.editingFinished.connect(self._distChanged)
        self._qcbType.currentIndexChanged.connect(self._geometryChanged)

    def __buildType(self):
        widget = qt.QWidget(self)
        widget.setLayout(qt.QHBoxLayout())
        # add QLabel
        widget.layout().addWidget(qt.QLabel("Reconstruction geometry :", widget))
        # build combobox
        self._qcbType = qt.QComboBox(widget)
        self.dic = {"parallel": "p", "conical": "c", "fan beam": "f"}
        for key in self.dic.keys():
            self._qcbType.addItem(key)
        self.linkComboboxWithH5Variable(self._qcbType, "TYPE", dic=self.dic)
        widget.layout().addWidget(self._qcbType)

        return widget

    def _typeChanged(self):
        if self._isLoading is False:
            value = self.dic[self._qcbType.currentText()]
            self._recons_params._set_parameter_value(parameter="TYPE", value=value)

    def __buildSXSYDist(self):
        self._geoOptsWidget = qt.QWidget(self)
        self._geoOptsWidget.setLayout(qt.QGridLayout())

        # SX
        self._qleSX = qt.QLineEdit("", parent=self._geoOptsWidget)
        self.LinkLineEditWithH5Variable(self._qleSX, "SX", float)
        validator = qt.QDoubleValidator(parent=self._qleSX)
        self._qleSX.setValidator(validator)
        self._geoOptsWidget.layout().addWidget(self._qleSX, 0, 1)
        self._geoOptsWidget.layout().addWidget(
            qt.QLabel(
                "Source position on vertical axis X (mm)", parent=self._geoOptsWidget
            ),
            0,
            0,
        )

        # SY
        self._qleSY = qt.QLineEdit("", parent=self._geoOptsWidget)
        self.LinkLineEditWithH5Variable(self._qleSY, "SY", float)
        validator = qt.QDoubleValidator(parent=self._qleSY)
        self._qleSY.setValidator(validator)
        self._geoOptsWidget.layout().addWidget(self._qleSY, 1, 1)
        self._geoOptsWidget.layout().addWidget(
            qt.QLabel(
                "Source position on vertical axis Y (mm)", parent=self._geoOptsWidget
            ),
            1,
            0,
        )

        # DIST
        self._qleDIST = qt.QLineEdit("", parent=self._geoOptsWidget)
        self.LinkLineEditWithH5Variable(self._qleDIST, "DIST", float)
        validator = qt.QDoubleValidator(parent=self._qleDIST)
        self._qleDIST.setValidator(validator)
        self._geoOptsWidget.layout().addWidget(self._qleDIST, 2, 1)
        self._geoOptsWidget.layout().addWidget(
            qt.QLabel("Source distance (m)", parent=self._geoOptsWidget), 2, 0
        )

        return self._geoOptsWidget

    def _SXChanged(self):
        if self._isLoading is False:
            value = float(self._qleSX.text())
            self._recons_params._set_parameter_value(parameter="SX", value=value)

    def _SYChanged(self):
        if self._isLoading is False:
            value = float(self._qleSY.text())
            self._recons_params._set_parameter_value(parameter="SY", value=value)

    def _distChanged(self):
        if self._isLoading is False:
            value = float(self._qleDIST.text())
            self._recons_params._set_parameter_value(parameter="DIST", value=value)

    def _geometryChanged(self, *args, **kwargs):
        self._geoOptsWidget.setVisible(self._qcbType.currentText() != "parallel")



# coding: utf-8
# /*##########################################################################
#
# Copyright (c) 2016-2017 European Synchrotron Radiation Facility
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# ###########################################################################*/

__authors__ = ["C. Nemoz", "H. Payno"]
__license__ = "MIT"
__date__ = "25/10/2016"

from ..utils import WidgetLongProcessing
from orangewidget import widget, gui
from orangewidget.settings import Setting
from orangewidget.widget import Input, Output
from orangecontrib.tomwer.orange.settings import CallbackSettingsHandler
from tomwer.gui.reconstruction.ftserie import FtserieWidget
from tomwer.core.scan.scanbase import TomwerScanBase
from tomwer.core.scan.scanbase import _TomwerBaseDock
from tomwer.utils import docstring
import tomwer.core.process.reconstruction.ftseries.ftseries
from ...orange.managedprocess import SuperviseOW
from silx.gui import qt
import copy
import logging

logger = logging.getLogger(__name__)


class FtseriesOW(SuperviseOW, WidgetLongProcessing):
    name = "pyhst2 slice reconstruction"
    id = "orange.widgets.tomwer.ftseries"
    description = "This widget manage the reconstruction of acquisitions"
    icon = "icons/bricks.png"
    priority = 13
    keywords = ["tomography", "file", "tomwer", "pyhst2"]

    want_main_area = True
    resizing_enabled = True
    allows_cycle = True
    compress_signal = False

    ewokstaskclass = tomwer.core.process.reconstruction.ftseries.ftseries.Ftseries

    settingsHandler = CallbackSettingsHandler()

    _rpSetting = Setting(dict())
    # this one is keep for backward compatibility. Will be removed on 0.10 I guess

    static_input = Setting({"data": None, "pyhst2_params": None})

    sigScanReady = qt.Signal()
    """Signal emitted when scan is process"""

    class Inputs:
        recons_param_changed = Input(
            name="change recons params",
            type=_TomwerBaseDock,
        )

        data = Input(name="data", type=TomwerScanBase, doc="one scan to be process")

    class Outputs:
        data = Output(name="data", type=TomwerScanBase, doc="one scan to be process")

    def __init__(self, parent=None, _connect_handler=True, recons_params=None):
        """
        Widget which read the .hdf5 generated by octave and modify it.
        Then run a subprocess to call octave and run ftseries

        :param bool _connect_handler: True if we want to store the modifications
                                      on the setting. Need for unit test since
                                      keep alive qt widgets.
        :param recons_params: reconsparameter to be used by the FTWidget.
                              If None, some will be created.
        :type: :class:`QReconsParams`
        """
        SuperviseOW.__init__(self, parent)
        WidgetLongProcessing.__init__(self)

        # add the ftserie widget
        self._ftserie = FtserieWidget(
            recons_params=recons_params, process_id=self.process_id
        )
        self._ftserie.hideExecuteButton()

        pyhst2_params = self.static_input.get("pyhst2_params", None)
        if pyhst2_params is None:
            pyhst2_params = self._rpSetting
        if pyhst2_params != dict():
            try:
                self._ftserie.recons_params.load_from_dict(pyhst2_params)
            except Exception:
                logger.warning("fail to load reconstruction settings")

        self._ftserie.sigScanReady.connect(self._signalReconsReady)

        _layout = gui.vBox(self.mainArea, self.name).layout()
        _layout.addWidget(self._ftserie)

        # add default buttons
        types = qt.QDialogButtonBox.Ok | qt.QDialogButtonBox.Cancel
        self._controlButtons = qt.QDialogButtonBox(parent=self)
        self._controlButtons.setStandardButtons(types)
        _layout.addWidget(self._controlButtons)
        self._controlButtons.setVisible(False)

        # expose API
        self.updatePath = self._ftserie.updatePath
        self.load = self._ftserie.load_recons_params
        self.save = self._ftserie.save
        self.getFileEditor = self._ftserie.getReconsParamSetEditor
        self.setH5Exploration = self._ftserie.setH5Exploration
        self.setReconsParams = self._ftserie.set_recons_params

        # connect signal / slot
        if _connect_handler is True:
            self.settingsHandler.addCallback(self._updateSettingsVals)

        # signal / slot connections
        self._ftserie._reconstruction_stack.sigReconsStarted.connect(
            self.__processing_start
        )
        self._ftserie._reconstruction_stack.sigReconsFinished.connect(
            self.__processing_end
        )
        self._ftserie._reconstruction_stack.sigReconsFailed.connect(
            self.__processing_end
        )
        self._ftserie._reconstruction_stack.sigReconsFinished.connect(self._log_succeed)
        self._ftserie._reconstruction_stack.sigReconsFailed.connect(self._log_failed)
        self._ftserie._reconstruction_stack.sigReconsMissParams.connect(
            self.__processing_end
        )

        self._controlButtons.accepted.connect(self.accept)
        self._controlButtons.rejected.connect(self.reject_from_dialog)

    def _signalReconsReady(self, scan):
        assert isinstance(scan, TomwerScanBase)
        self.Outputs.data.send(scan)
        self.sigScanReady.emit()

    @Inputs.recons_param_changed
    def updateReconsParam(self, ftseries):
        self._ftserie.updateReconsParam(ftseries)
        self._controlButtons.setVisible(True)
        self.show()
        self.exec_()
        self._controlButtons.setVisible(False)

    @Inputs.data
    def pathReceived(self, scan):
        if scan is None:
            return
        if not isinstance(scan, TomwerScanBase):
            raise TypeError(f"{scan} is expected to be an instance of {TomwerScanBase}")
        scan_ = copy.copy(scan)
        scan_.clear_latest_reconstructions()

        self._ftserie.setScan(scan=scan_)
        self._ftserie.process(scan_)

    @docstring(SuperviseOW)
    def reprocess(self, dataset):
        self.pathReceived(dataset)

    def _updateSettingsVals(self):
        self._rpSetting = self._ftserie.recons_params.to_dict()
        # keep backward compatibility
        self.static_input = {
            "data": None,
            "pyhst2_params": self._ftserie.recons_params.to_dict(),
        }

    @property
    def recons_params(self):
        return self._ftserie.recons_params

    def accept(self):
        widget.OWBaseWidget.accept(self)
        self._ftserie.process(self.scan)

    def reject_from_dialog(self):
        widget.OWBaseWidget.reject(self)
        self._signalReconsReady(self.scan)

    def __processing_start(self, scan):
        logger.processStarted(
            "Start pyhst slice reconstruction of {}".format(str(scan))
        )
        self.processing_state(working=True, info=" ")

    def __processing_end(self, scan):
        self.processing_state(working=False, info=" ")

    def _log_succeed(self, scan):
        logger.processSucceed("Pyhst succeed to reconstruct {}".format(str(scan)))

    def _log_failed(self, scan):
        logger.processFailed("Pyhst failed to reconstruct {}".format(str(scan)))

    def close(self):
        self._ftserie._reconstruction_stack.stop()
        super(FtseriesOW, self).close()

    @property
    def scan(self):
        return self._ftserie.scan

    def load_recons_params(self, params):
        return self._ftserie.load_recons_params(params)

    def setDryRun(self, dry_run):
        self._ftserie.setDryRun(dry_run)



# coding: utf-8
# /*##########################################################################
# Copyright (C) 2016 European Synchrotron Radiation Facility
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
#############################################################################*/


from __future__ import absolute_import, division, print_function

"""Test suite to scripts"""

__authors__ = ["V.Valls", "H.Payno"]
__license__ = "MIT"
__copyright__ = "European Synchrotron Radiation Facility, Grenoble, France"
__date__ = "21/02/2018"


import sys
import unittest
import logging
import subprocess
from tomwer.test.utils.utilstest import UtilsTest

_logger = logging.getLogger(__name__)


class TestScriptsHelp(unittest.TestCase):
    def executeCommandLine(self, command_line, env):
        """Execute a command line.

        Log output as debug in case of bad return code.
        """
        _logger.info("Execute: %s", " ".join(command_line))
        p = subprocess.Popen(
            command_line, stdout=subprocess.PIPE, stderr=subprocess.PIPE, env=env
        )
        out, err = p.communicate()
        _logger.info("Return code: %d", p.returncode)
        try:
            out = out.decode("utf-8")
        except UnicodeError:
            pass
        try:
            err = err.decode("utf-8")
        except UnicodeError:
            pass

        if p.returncode != 0:
            _logger.info("stdout:")
            _logger.info("%s", out)
            _logger.info("stderr:")
            _logger.info("%s", err)
        else:
            _logger.debug("stdout:")
            _logger.debug("%s", out)
            _logger.debug("stderr:")
            _logger.debug("%s", err)
        self.assertEqual(p.returncode, 0)

    def executeAppHelp(self, script_name, module_name):
        script = UtilsTest.script_path(script_name, module_name)
        env = UtilsTest.get_test_env()
        if script.endswith(".exe"):
            command_line = [script]
        else:
            command_line = [sys.executable, script]
        command_line.append("--help")
        self.executeCommandLine(command_line, env)

    def testAxis(self):
        self.executeAppHelp("axis", "tomwer.app.axis")

    def testDarkRef(self):
        self.executeAppHelp("darkref", "tomwer.app.darkref")

    def testFtserie(self):
        self.executeAppHelp("ftserie", "tomwer.app.ftseries")

    def testLamino(self):
        self.executeAppHelp("lamino", "tomwer.app.lamino")

    def testNabu(self):
        self.executeAppHelp("nabu", "tomwer.app.nabuapp")

    def testRadioStack(self):
        self.executeAppHelp("radiostack", "tomwer.app.radiostack")

    def testSampleMoved(self):
        self.executeAppHelp("samplemoved", "tomwer.app.samplemoved")

    def testSliceStack(self):
        self.executeAppHelp("slicestack", "tomwer.app.slicestack")

    def testRSync(self):
        self.executeAppHelp("rsync", "tomwer.app.rsync")
